#define SPATHI_ICON_MASK_PMAP_ANIM 0x00200002L
#define SPATHI_MICON_MASK_PMAP_ANIM 0x00200102L
#define SPATHI_BIG_MASK_PMAP_ANIM 0x00400202L
#define SPATHI_MED_MASK_PMAP_ANIM 0x00400302L
#define SPATHI_SML_MASK_PMAP_ANIM 0x00400402L
#define MISSILE_BIG_MASK_PMAP_ANIM 0x00400502L
#define MISSILE_MED_MASK_PMAP_ANIM 0x00400602L
#define MISSILE_SML_MASK_PMAP_ANIM 0x00400702L
#define DISCRIM_BIG_MASK_PMAP_ANIM 0x00400802L
#define DISCRIM_MED_MASK_PMAP_ANIM 0x00400902L
#define DISCRIM_SML_MASK_PMAP_ANIM 0x00400a02L
#define SPATHI_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


