//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	NEUTRAL_SPACE_HELLO_1,
	NEUTRAL_SPACE_HELLO_2,
	HOSTILE_SPACE_HELLO_1,
	HOSTILE_SPACE_HELLO_2,
	BOMB_WORLD_HELLO_1,
	BOMB_WORLD_HELLO_2,
	HOSTILE_BOMB_HELLO_1,
	HOSTILE_BOMB_HELLO_2,
	NEUTRAL_HOMEWORLD_HELLO_1,
	NEUTRAL_HOMEWORLD_HELLO_2,
	NEUTRAL_HOMEWORLD_HELLO_3,
	NEUTRAL_HOMEWORLD_HELLO_4,
	HOSTILE_HOMEWORLD_HELLO_1,
	HOSTILE_HOMEWORLD_HELLO_2,
	why_you_here,
	WE_GUARD_BOMB,
	what_about_bomb,
	ABOUT_BOMB,
	give_us_bomb_or_die,
	GUARDS_WARN,
	demand_bomb,
	GUARDS_FIGHT,
	may_we_have_bomb,
	NO_BOMB,
	please,
	SORRY_NO_BOMB,
	whats_up_bomb,
	GENERAL_INFO_BOMB_1,
	GENERAL_INFO_BOMB_2,
	bye_bomb,
	GOODBYE_BOMB,
	hey_wait_got_ultron,
	TAUNT_US_BUT_WE_LOOK,
	TRICKED_US_1,
	TRICKED_US_2,
	we_are_vindicator0,
	we_are_vindicator1,
	we_are_vindicator2,
	WOULD_BE_HAPPY_BUT,
	why_sad,
	ULTRON_BROKE,
	what_ultron,
	GLORIOUS_ULTRON,
	dont_be_babies,
	MOCK_OUR_PAIN,
	real_sorry_about_ultron,
	APPRECIATE_SYMPATHY,
	what_about_you_1,
	ABOUT_US_1,
	what_about_you_2,
	ABOUT_US_2,
	what_about_you_3,
	ABOUT_US_3,
	what_about_urquan_1,
	ABOUT_URQUAN_1,
	what_about_urquan_2,
	ABOUT_URQUAN_2,
	got_ultron,
	DONT_WANT_TO_LOOK,
	SICK_TRICK_1,
	SICK_TRICK_2,
	bye_neutral,
	GOODBYE_NEUTRAL,
	TOO_LATE,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	HAPPY_DAYS,
	OK_ATTACK_KOHRAH,
	whats_up_after_space,
	GENERAL_INFO_AFTER_SPACE_1,
	GENERAL_INFO_AFTER_SPACE_2,
	what_now_after_space,
	DO_THIS_AFTER_SPACE,
	bye_after_space,
	GOODBYE_AFTER_SPACE,
	whats_up_before_space,
	GENERAL_INFO_BEFORE_SPACE_1,
	GENERAL_INFO_BEFORE_SPACE_2,
	what_now_before_space,
	DO_THIS_BEFORE_SPACE,
	bye_before_space,
	GOODBYE_BEFORE_SPACE,
	how_went_war,
	ABOUT_BATTLE,
	how_goes_war,
	BATTLE_HAPPENS_1,
	BATTLE_HAPPENS_2,
	FLEET_ON_WAY,
	learn_new_info,
	NO_NEW_INFO,
	SAMATRA,
	what_now_homeworld,
	HOPE_KILL_EACH_OTHER,
	how_is_ultron,
	ULTRON_IS_GREAT,
	bye_allied_homeworld,
	GOODBYE_ALLIED_HOMEWORLD,
	ALLIED_HOMEWORLD_HELLO_1,
	ALLIED_HOMEWORLD_HELLO_2,
	ALLIED_HOMEWORLD_HELLO_3,
	ALLIED_HOMEWORLD_HELLO_4,
	HELLO_BEFORE_KOHRAH_SPACE_1,
	HELLO_BEFORE_KOHRAH_SPACE_2,
	HELLO_DURING_KOHRAH_SPACE_1,
	HELLO_DURING_KOHRAH_SPACE_2,
	HELLO_AFTER_KOHRAH_SPACE_1,
	HELLO_AFTER_KOHRAH_SPACE_2,
	UP_TO_YOU,
	can_you_help,
	HOW_HELP,
	DONT_NEED,
	HAVE_4_SHIPS,
	NO_ULTRON_AT_BOMB,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
