//Copyright Paul Reiche, Fred Ford. 1992-2002
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	NEVER_ENOUGH,
	OK_WARSHIP,
	OK_DWE,
	OK_YOUBOO,
	OK_DILRAT,
	BIG_FUN,
	FAST_AS_CAN,
	GLORIOUS_WORSHIP,
	ON_WAY,
	GODS_RETURN_1,
	GODS_RETURN_2,
	GODS_RETURN_3,
	JUST_GRUNTS,
	GRUNTS_AGAIN,
	WHAT_ORDERS,
	WE_WORSHIP_1,
	WE_WORSHIP_2,
	WE_WORSHIP_3,
	SUBSEQUENT_CHMMR_HELLO,
	INIT_CHMMR_HELLO,
	OK_ENOUGH_ILWRATH,
	OK_ENOUGH_GODS,
	SEND_MESSAGE,
	CAME_FROM,
	WHO_BLASTS_WHO,
	NO_SURRENDER,
	NOT_REASONABLE,
	SUBSEQUENT_HOME_HELLO,
	GENERAL_INFO,
	GOODBYE_AND_DIE,
	DECEIVERS,
	NO_PEACE,
	NO_ALLIANCE,
	ILWRATH_BELIEVE,
	OK_KILL_THRADDASH,
	GOODBYE_GODS,
	INIT_HELLO_SPACE,
	SUBSEQUENT_HELLO_SPACE_1,
	SUBSEQUENT_HELLO_SPACE_2,
	SUBSEQUENT_HELLO_SPACE_3,
	SUBSEQUENT_HELLO_SPACE_4,
	GENERAL_INFO_SPACE_1,
	GENERAL_INFO_SPACE_2,
	GENERAL_INFO_SPACE_3,
	GENERAL_INFO_SPACE_4,
	GENERAL_INFO_SPACE_5,
	STRENGTH_NOT_ALL,
	NO_SLAY_BY_THOUSANDS,
	NO_EASE_UP,
	GOODBYE_AND_DIE_SPACE,
	INIT_HOME_HELLO,
	GOODBYE_AND_DIE_HOMEWORLD,
	SO_MUCH_TO_KNOW,
	LONG_AGO,
	KILLED_GOOD_GODS,
	CHANNEL_44,
	BECAUSE_44,
	WHAT_ABOUT_ILWRATH,
	ABOUT_PHYSIO,
	ABOUT_HISTORY,
	ABOUT_CULTURE,
	ABOUT_URQUAN,
	URQUAN_TOO_NICE,
	OF_COURSE_WERE_EVIL,
	DONT_CONFUSE_US,
	ON_WAY_TO_THRADDASH,
	HAPPY_FIGHTING_THRADDASH,

	say_warship,
	say_dwe,
	say_youboo,
	say_dillrat,
	enough_orders,
	other_divine_orders,
	worship_us,
	bye_gods,
	enough_ilwrath,
	enough_gods,
	where_you_come_from,
	it_will_be_a_pleasure,
	be_reasonable,
	surrender,
	whats_up,
	bye,
	want_peace,
	want_alliance,
	go_kill_thraddash,
	whats_up_space_1,
	whats_up_space_2,
	whats_up_space_3,
	whats_up_space_4,
	whats_up_space_5,
	you_are_weak,
	slay_by_thousands,
	ease_up,
	bye_space,
	bye_homeworld,
	want_info_on_gods,
	when_start_worship,
	any_good_gods,
	how_talk_with_gods,
	why_44,
	want_info_on_ilwrath,
	what_about_physio,
	what_about_history,
	what_about_culture,
	what_about_urquan,
	are_you_evil,
	but_evil_is_defined,
};

#endif /* _STRINGS_H */
