//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	WE_COME_IN_PEACE_1,
	WE_COME_IN_PEACE_2,
	WE_COME_IN_PEACE_3,
	WE_COME_IN_PEACE_4,
	WE_COME_IN_PEACE_5,
	WE_COME_IN_PEACE_6,
	WE_COME_IN_PEACE_7,
	WE_COME_IN_PEACE_8,
	threat_1,
	threat_2,
	threat_3,
	threat_4,
	PROGRAMMED_TO_DEFEND_1,
	PROGRAMMED_TO_DEFEND_2,
	PROGRAMMED_TO_DEFEND_3,
	PROGRAMMED_TO_DEFEND_4,
	something_wrong_1,
	something_wrong_2,
	something_wrong_3,
	something_wrong_4,
	NOMINAL_FUNCTION_1,
	NOMINAL_FUNCTION_2,
	NOMINAL_FUNCTION_3,
	NOMINAL_FUNCTION_4,
	we_are_us_1,
	we_are_us_2,
	we_are_us_3,
	we_are_us_4,
	THIS_IS_PROBE_1,
	THIS_IS_PROBE_2,
	THIS_IS_PROBE_3,
	THIS_IS_PROBE_40,
	THIS_IS_PROBE_41,
	THIS_IS_PROBE_42,
	why_attack_1,
	why_attack_2,
	why_attack_3,
	why_attack_4,
	PEACEFUL_MISSION_1,
	PEACEFUL_MISSION_2,
	PEACEFUL_MISSION_3,
	PEACEFUL_MISSION_4,
	bye_1,
	bye_2,
	bye_3,
	bye_4,
	GOODBYE_1,
	GOODBYE_2,
	GOODBYE_3,
	GOODBYE_4,
	HOSTILE,
	DESTRUCT_SEQUENCE,
	destruct_code,
};

#endif /* _STRINGS_H */
