//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HELLO_AT_UMGAH,
	what_are_you,
	JUST_TALKING_PET,
	talking_pets_dumb,
	OH_NO_YOU_DONT,
	what_do_to_umgah,
	DID_NOTHING,
	umgah_zombies,
	WORKS_LIKE_THIS,
	we_are_vindicator0,
	we_are_vindicator1,
	GOOD_FOR_YOU,
	must_explain_presence,
	EXPLAIN_NOTHING_MONKEY_BOY,
	bye_at_umgah,
	GOODBYE_AT_UMGAH,
	HYPNOTIZE_AGAIN_1,
	HYPNOTIZE_AGAIN_2,
	HYPNOTIZE_AGAIN_3,
	HYPNOTIZE_AGAIN_4,
	HYPNO_TAIL,
	CANT_COMPEL,
	LETS_MAKE_A_DEAL,
	what_kind_of_deal,
	HELP_DEFEAT_URQUAN,
	ok_lets_do_it,
	COMING_ABOARD,
	how_trust,
	TRUST,
	boneless_dweeb,
	YOUR_BONELESS_DWEEB,
	what_are_you_really,
	POOR_DNYARRI,
	hard_to_believe,
	ITS_TRUE,
	bullshit,
	WORTH_A_TRY,
	kill_you,
	PLEASE_DONT,
	must_kill,
	DONT_KILL,
	want_kill_1,
	want_kill_2,
	want_kill_3,
	GLAD_YOU_WONT_KILL,
	whats_up_onboard,
	GENERAL_INFO_ONBOARD_1,
	GENERAL_INFO_ONBOARD_2,
	GENERAL_INFO_ONBOARD_3,
	GENERAL_INFO_ONBOARD_4,
	GENERAL_INFO_ONBOARD_5,
	GENERAL_INFO_ONBOARD_6,
	GENERAL_INFO_ONBOARD_7,
	GENERAL_INFO_ONBOARD_8,
	HELLO_AS_DEVICE_1,
	HELLO_AS_DEVICE_2,
	HELLO_AS_DEVICE_3,
	HELLO_AS_DEVICE_4,
	HELLO_AS_DEVICE_5,
	HELLO_AS_DEVICE_6,
	HELLO_AS_DEVICE_7,
	HELLO_AS_DEVICE_8,
	CYBORG_PEP_TALK,
	HUMAN_PEP_TALK,
	I_SENSE_MY_SLAVES,
	HAVENT_GOT_EVERYTHING,
	NEED_BOMB,
	SOUP_UP_BOMB,
	SOUP_UP_FLEET,
	SOUP_UP_FLAGSHIP,
	COMEBACK_WHEN_READY,
	what_now,
	DO_THIS,
	compel_urquan,
	HERE_WE_GO,
	im_scared,
	STUPID_FOP,
	compel_that_ship,
	SAVING_MY_POWER,
	any_suggestions,
	SUGGESTION_1,
	SUGGESTION_2,
	SUGGESTION_3,
	SUGGESTION_4,
	SUGGESTION_5,
	SUGGESTION_6,
	SUGGESTION_7,
	SUGGESTION_8,
	about_your_race,
	WHAT_ABOUT_RACE,
	you_lied,
	SO_WHAT,
	bye_onboard,
	GOODBYE_ONBOARD,
	what_about_physiology,
	NO_TALK_ABOUT_SELF,
	what_about_powers,
	NOT_POWERS_BUT_FLOWERS,
	yes_flowers,
	GOOD_HUMAN,
	wish_to_go_now,
	EXCELLENT_IDEA,
	what_about_your_history,
	ABOUT_HISTORY,
	sentient_milieu,
	ABOUT_SENTIENT_MILIEU,
	what_about_war,
	ABOUT_WAR,
	enough_info,
	OK_ENOUGH_INFO,
	UMGAH_ALL_GONE,
	HELLO_AFTER_COMPEL_URQUAN,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
