//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	SENSE_EVIL,
	HELLO_AND_DIE_1,
	HELLO_AND_DIE_2,
	HELLO_AND_DIE_3,
	HELLO_AND_DIE_4,
	HELLO_AND_DIE_5,
	HELLO_AND_DIE_6,
	HELLO_AND_DIE_7,
	HELLO_AND_DIE_8,
	HELLO_SAMATRA,
	WE_KILL_ALL_1,
	WE_KILL_ALL_2,
	WE_KILL_ALL_3,
	WE_KILL_ALL_4,
	why_kill_all_1,
	why_kill_all_2,
	why_kill_all_3,
	why_kill_all_4,
	KILL_BECAUSE_1,
	KILL_BECAUSE_2,
	KILL_BECAUSE_3,
	KILL_BECAUSE_4,
	please_dont_kill_1,
	WILL_KILL_1,
	please_dont_kill_2,
	WILL_KILL_2,
	please_dont_kill_3,
	WILL_KILL_3,
	please_dont_kill_4,
	WILL_KILL_4,
	bye_frenzy_1,
	bye_frenzy_2,
	bye_frenzy_3,
	bye_frenzy_4,
	GOODBYE_AND_DIE_FRENZY_1,
	GOODBYE_AND_DIE_FRENZY_2,
	GOODBYE_AND_DIE_FRENZY_3,
	GOODBYE_AND_DIE_FRENZY_4,
	threat_1,
	RESISTANCE_IS_USELESS_1,
	threat_2,
	RESISTANCE_IS_USELESS_2,
	threat_3,
	RESISTANCE_IS_USELESS_3,
	threat_4,
	RESISTANCE_IS_USELESS_4,
	key_phrase,
	RESPONSE_TO_KEY_PHRASE,
	why_do_you_destroy,
	WE_WERE_SLAVES,
	relationship_with_urquan,
	WE_ARE_URQUAN_TOO,
	what_about_culture,
	BONE_GARDENS,
	how_leave_me_alone,
	YOU_DIE,
	guess_thats_all,
	THEN_DIE,
	what_are_you_hovering_over,
	BONE_PILE,
	you_sure_are_creepy,
	YES_CREEPY,
	stop_that_gross_blinking,
	DIE_HUMAN,
	plead_1,
	PLEADING_IS_USELESS_1,
	plead_2,
	PLEADING_IS_USELESS_2,
	plead_3,
	PLEADING_IS_USELESS_3,
	plead_4,
	PLEADING_IS_USELESS_4,
	bye,
	GOODBYE_AND_DIE,
	GAME_OVER_DUDE,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
