#define CHMMR_ICON_MASK_PMAP_ANIM 0x00200002L
#define CHMMR_MICON_MASK_PMAP_ANIM 0x00200102L
#define CHMMR_BIG_MASK_PMAP_ANIM 0x00400202L
#define CHMMR_MED_MASK_PMAP_ANIM 0x00400302L
#define CHMMR_SML_MASK_PMAP_ANIM 0x00400402L
#define MUZZLE_BIG_MASK_PMAP_ANIM 0x00400502L
#define MUZZLE_MED_MASK_PMAP_ANIM 0x00400602L
#define MUZZLE_SML_MASK_PMAP_ANIM 0x00400702L
#define SATELLITE_BIG_MASK_PMAP_ANIM 0x00400802L
#define SATELLITE_MED_MASK_PMAP_ANIM 0x00400902L
#define SATELLITE_SML_MASK_PMAP_ANIM 0x00400a02L
#define CHMMR_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


