#define ILWRATH_ICON_MASK_PMAP_ANIM 0x00200002L
#define ILWRATH_MICON_MASK_PMAP_ANIM 0x00200102L
#define ILWRATH_BIG_MASK_PMAP_ANIM 0x00400202L
#define ILWRATH_MED_MASK_PMAP_ANIM 0x00400302L
#define ILWRATH_SML_MASK_PMAP_ANIM 0x00400402L
#define FIRE_BIG_MASK_PMAP_ANIM 0x00400502L
#define FIRE_MED_MASK_PMAP_ANIM 0x00400602L
#define FIRE_SML_MASK_PMAP_ANIM 0x00400702L
#define ILWRATH_CAPTAIN_MASK_PMAP_ANIM 0x00400802L


