#define VUX_ICON_MASK_PMAP_ANIM 0x00200002L
#define VUX_MICON_MASK_PMAP_ANIM 0x00200102L
#define VUX_BIG_MASK_PMAP_ANIM 0x00400202L
#define VUX_MED_MASK_PMAP_ANIM 0x00400302L
#define VUX_SML_MASK_PMAP_ANIM 0x00400402L
#define LIMPETS_BIG_MASK_PMAP_ANIM 0x00400502L
#define LIMPETS_MED_MASK_PMAP_ANIM 0x00400602L
#define LIMPETS_SML_MASK_PMAP_ANIM 0x00400702L
#define SLIME_MASK_PMAP_ANIM 0x00400802L
#define VUX_CAPTAIN_MASK_PMAP_ANIM 0x00400902L


