#define SHOFIXTI_ICON_MASK_PMAP_ANIM 0x00200002L
#define SHOFIXTI_MICON_MASK_PMAP_ANIM 0x00200102L
#define SHOFIXTI_BIG_MASK_PMAP_ANIM 0x00400202L
#define SHOFIXTI_MED_MASK_PMAP_ANIM 0x00400302L
#define SHOFIXTI_SML_MASK_PMAP_ANIM 0x00400402L
#define DESTRUCT_BIG_MASK_ANIM 0x00400502L
#define DESTRUCT_MED_MASK_ANIM 0x00400602L
#define DESTRUCT_SML_MASK_ANIM 0x00400702L
#define SHOFIXTI_CAPTAIN_MASK_PMAP_ANIM 0x00400802L
#define OLDSHOF_BIG_MASK_PMAP_ANIM 0x00600902L
#define OLDSHOF_MED_MASK_PMAP_ANIM 0x00600a02L
#define OLDSHOF_SML_MASK_PMAP_ANIM 0x00600b02L
#define OLDSHOF_CAPTAIN_MASK_PMAP_ANIM 0x00600c02L
#define DART_BIG_MASK_PMAP_ANIM 0x00800d02L
#define DART_MED_MASK_PMAP_ANIM 0x00800e02L
#define DART_SML_MASK_PMAP_ANIM 0x00800f02L


