//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HELLO_1,
	HELLO_2,
	HELLO_3,
	HELLO_4,
	RECALL_PROGRAM_1,
	we_are_us0,
	we_are_us1,
	we_are_us2,
	TERRIBLY_EXCITING,
	happy_to_tell_more,
	TELL_MORE,
	would_you_like_to_know_more,
	YES_TELL_MORE,
	we_come_from_earth,
	OK_EARTH,
	we_explore,
	OK_EXPLORE,
	we_fight_urquan,
	URQUAN_NICE_GUYS,
	not_same_urquan,
	PERSONALITY_CHANGE,
	we_gather,
	MAYBE_INTERESTED,
	enough_about_me,
	OK_ENOUGH_YOU,
	what_other_visitors,
	VISITORS,
	any_other_visitors,
	LONG_AGO,
	what_about_sentient_milieu,
	MET_TAALO_THEY_ARE_FROM,
	who_else,
	PRECURSORS,
	precursors_yow,
	ABOUT_PRECURSORS,
	must_know_more,
	ALL_WE_KNOW,
	who_are_you,
	WE_ARE_SLY,
	like_more_about_you,
	SURE_KNOW_WHAT,
	what_about_home,
	ABOUT_HOME,
	what_about_culture,
	ABOUT_CULTURE,
	what_about_history,
	ABOUT_HISTORY,
	what_about_biology,
	ABOUT_BIOLOGY,
	enough_info,
	OK_ENOUGH_INFO,
	where_are_you,
	DOWN_HERE,
	thats_impossible_1,
	NO_ITS_NOT_1,
	thats_impossible_2,
	NO_ITS_NOT_2,
	bye,
	GOODBYE_1,
	GOODBYE_2,
	what_are_probes,
	PROBES_ARE,
	know_more_probe,
	OK_WHAT_MORE_PROBE,
	where_probes_from,
	PROBES_FROM_MELNORME,
	why_sell,
	SELL_FOR_INFO,
	how_long_ago,
	FIFTY_THOUSAND_ROTATIONS,
	whats_probes_mission,
	SEEK_OUT_NEW_LIFE,
	if_only_one,
	THEY_REPLICATE,
	enough_probe,
	OK_ENOUGH_PROBE,
	why_probe_always_attack,
	ONLY_DEFEND,
	talk_more_probe_attack,
	NO_PROBLEM_BUT_SURE,
	tell_me_about_basics,
	BASIC_COMMANDS,
	tell_basics_again,
	OK_BASICS_AGAIN,
	what_effect,
	AFFECTS_BEHAVIOR,
	how_does_probe_defend,
	ONLY_SELF_DEFENSE,
	combat_behavior,
	MISSILE_BATTERIES,
	what_missile_batteries,
	LIGHTNING_ONLY_FOR_HARVESTING,
	tell_me_about_rep_1,
	ABOUT_REP,
	what_set_priority,
	MAXIMUM,
	enough_problem,
	OK_ENOUGH_PROBLEM,
	probe_has_bug,
	NO_IT_DOESNT,
	tell_me_about_attack,
	ATTACK_NO_PROBLEM,
	tell_me_about_rep_2,
	REP_NO_PROBLEM,
	what_about_rep_priorities,
	MAXIMUM_SO_WHAT,
	think_about_rep_priorities,
	UH_OH,
	hunt_them_down,
	GROW_TOO_FAST,
	sue_melnorme,
	SIGNED_WAIVER,
	recall_signal,
	NOT_THIS_MODEL,
	mega_self_destruct,
	WHY_YES_THERE_IS,
};

#endif /* _STRINGS_H */
