//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	tanaka,
	katana,
	HOSTILE_KATANA_1,
	HOSTILE_KATANA_2,
	HOSTILE_KATANA_3,
	HOSTILE_KATANA_4,
	HOSTILE_TANAKA_1,
	HOSTILE_TANAKA_2,
	HOSTILE_TANAKA_3,
	HOSTILE_TANAKA_4,
	HOSTILE_TANAKA_5,
	HOSTILE_TANAKA_6,
	HOSTILE_TANAKA_7,
	HOSTILE_TANAKA_8,
	dont_attack,
	TYPICAL_PLOY,
	hey_stop,
	ONLY_STOP,
	look_you_are,
	TOO_BAD,
	dont_know,
	NEVER,
	look0,
	look1,
	FOR_YOU,
	no_bloodshed,
	YES_BLOODSHED,
	dont_want_to_fight,
	MUST_FIGHT_YOU_URQUAN_1,
	MUST_FIGHT_YOU_URQUAN_2,
	MUST_FIGHT_YOU_URQUAN_3,
	MUST_FIGHT_YOU_URQUAN_4,
	no_one_insults,
	YOU_LIMP,
	mighty_words,
	HANG_YOUR,
	donkey_breath,
	DGRUNTI,
	i_am_captain0,
	i_am_captain1,
	i_am_captain2,
	i_am_captain3,
	i_am_nice,
	i_am_guy,
	SO_SORRY,
	MUST_UNDERSTAND,
	NICE_BUT_WHAT_IS_DONKEY,
	IS_DEFEAT_TRUE,
	yes_and_no,
	butt_blasted,
	clobbered,
	VERY_SAD_KILL_SELF,
	important_duty,
	WHAT_DUTY,
	need_you_for_duty,
	OK_WILL_BE_SENTRY,
	dont_do_it,
	YES_I_DO_IT,
	go_ahead,
	ON_SECOND_THOUGHT,
	procreating_wildly,
	replenishing_your_species,
	hope_you_have,
	SOUNDS_GREAT_BUT_HOW,
	females,
	nubiles,
	rat_babes,
	LEAPING_HAPPINESS,
	bye0,
	bye1,
	GOODBYE0,
	GOODBYE1,
	why_here0,
	why_here1,
	I_GUARD,
	where_world,
	BLEW_IT_UP,
	how_survive,
	NOT_HERE,
	what_happened,
	MET_VUX,
	glory_device,
	SWITCH_BROKE,
	bye,
	GOODBYE,
	FRIENDLY_HELLO,
	report0,
	report1,
	NOTHING_NEW,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
