#define ORZ_ICON_MASK_PMAP_ANIM 0x00200002L
#define ORZ_MICON_MASK_PMAP_ANIM 0x00200102L
#define ORZ_BIG_MASK_PMAP_ANIM 0x00400202L
#define ORZ_MED_MASK_PMAP_ANIM 0x00400302L
#define ORZ_SML_MASK_PMAP_ANIM 0x00400402L
#define HOWITZER_BIG_MASK_PMAP_ANIM 0x00400502L
#define HOWITZER_MED_MASK_PMAP_ANIM 0x00400602L
#define HOWITZER_SML_MASK_PMAP_ANIM 0x00400702L
#define TURRET_BIG_MASK_PMAP_ANIM 0x00400802L
#define TURRET_MED_MASK_PMAP_ANIM 0x00400902L
#define TURRET_SML_MASK_PMAP_ANIM 0x00400a02L
#define ORZ_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


