//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	GLAD_WHEN_YOU_COME_BACK,
	GIVE_FUEL,
	GIVE_FUEL_AGAIN,
	ARE_YOU_SUPPLY_SHIP,
	DO_YOU_HAVE_RADIO_THIS_TIME,
	HERE_IS_ANOTHER_LANDER,
	THE_WHAT_FROM_WHERE,
	ABOUT_TIME,
	MESSAGE_GARBLED_1,
	MESSAGE_GARBLED_2,
	HERE_IS_A_NEW_LANDER,
	THIS_MAY_SEEM_SILLY,
	OK_THE_NAFS,
	OK_THE_CAN,
	OK_THE_UFW,
	OK_THE_NAME_IS_EMPIRE0,
	OK_THE_NAME_IS_EMPIRE1,
	FUEL_UP0,
	FUEL_UP1,
	WHAT_KIND_OF_IDIOT,
	DONT_KNOW_WHO_YOU_ARE,
	THATS_IMPOSSIBLE,
	ASK_AWAY,
	RADIOS_ON_MERCURY,
	THANKS_FOR_HELPING,
	STARBASE_IS,
	HAPPENED_TO_EARTH,
	URQUAN_LEFT,
	BASE_ON_MOON,
	ACKNOWLEDGE_SECRET,
	ABOUT_BASE,
	GOOD_LUCK_WITH_BASE,
	DEALT_WITH_BASE_YET,
	HERE_COMES_ILWRATH,
	VERY_IMPRESSIVE,
	IT_WAS_ABANDONED,
	YOU_REALLY_FOUGHT_BASE,
	IM_GLAD_YOU_WON,
	IM_SURE_IT_WAS_DIFFICULT,
	THAT_WAS_PROBE,
	DEEP_TROUBLE,
	GOOD_NEWS,
	SURE_HOPE,
	ABOUT_BASE_AGAIN,
	COOK_BUTTS,
	OVERTHROW_ALIENS,
	KILL_MONSTERS,
	GOOD_LUCK_AGAIN,
	STARBASE_WILL_BE_READY,

	overthrow_evil_aliens,
	annihilate_those_monsters,
	cook_their_butts,
	where_get_radios,
	well_go_get_them_now,
	we_will_transfer_now,
	what_will_you_give_us,
	before_radios_we_need_info,
	no_but_well_help0,
	no_but_well_help1,
	yes_this_is_supply_ship,
	what_slave_planet,
	i_lied,
	plumb_out,
	we_are_vindicator0,
	we_are_vindicator1,
	we_are_vindicator2,
	first_give_info,
	we_must_go_now,
	where_can_i_get_radios,
	ok_i_will_get_radios,
	whats_this_starbase,
	what_about_earth,
	where_are_urquan,
	our_mission_was_secret,
	we_are_here_to_help,
	tell_me_about_base,
	we_will_take_care_of_base,
	tell_me_again,
	base_was_abandoned,
	we_fought_them,
	oh_yes_big_fight,
	i_lied_it_was_abandoned,
	i_cant_talk_about_it,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	i_lost_my_lander,
	i_lost_another_lander,
	need_fuel_mercury,
	need_fuel_luna,
	need_fuel_again,
	what_was_red_thing,
	it_went_away,
	we_destroyed_it,
	what_probe,
	take_care_of_base_again,
	goodbye_commander,
};

#endif /* _STRINGS_H */
