//Copyright Paul Reiche, Fred Ford. 1992-2002
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	SORRY_ABOUT_THAT,
	identify,
	I_FWIFFO,
	hi_there,
	ARE_YOU_SURE,
	dont_kill,
	we_fight_1,
	we_fight_2,
	OK_WONT,
	do_cultural,
	WEZZY_WEZZAH,
	die_slugboy,
	begin_ritual,
	MUST_DO_RITUAL_AT_HOME,
	you_wont_die_yet,
	ETERNAL_GRATITUDE,
	we_fight,
	pay_for_crimes,
	CLUTCH_MAVEN,
	you_may_live,
	HONEST_AND_FRIENDLY,
	what_are_coordinates,
	COORDINATES_ARE,
	tell_me_coordinates,
	FAKE_COORDINATES,
	TOO_SCARY,
	youve_got_me_all_wrong,
	SORRY_NO_COORDS,
	what_doing_on_pluto_1,
	ABOUT_20_YEARS_AGO,
	what_doing_on_pluto_2,
	WHEN_URQUAN_ARRIVED,
	where_are_urquan,
	URQUAN_LEFT,
	what_about_other_races,
	ABOUT_OTHER_RACES,
	what_doing_on_pluto_3,
	what_about_yourself,
	ABOUT_MYSELF,
	STATIONED_ON_EARTH_MOON,
	what_blaze_of_glory,
	BLAZE_IS,
	what_about_moonbase,
	SET_UP_BASE,
	what_about_ilwrath,
	ABOUT_ILWRATH,
	what_about_other_spathi,
	really_thousands,
	SPATHI_ARE,
	what_enemy,
	ENEMY_IS,
	when_ilwrath,
	THEN_ILWRATH,
	why_you_here,
	DREW_SHORT_STRAW,
	how_many_crew,
	JUST_ME,
	THOUSANDS,
	full_of_monsters,
	HOW_TRUE,
	join_us,
	WILL_JOIN,
	WONT_JOIN_1,
	give_ship_or_die,
	WONT_JOIN_2,
	WONT_JOIN_3,
	GEE_THANKS,
	changed_mind,
	youre_forgiven,
	THANKS_FOR_FORGIVENESS,
	HATE_YOU_FOREVER_SPACE,
	INIT_ANGRY_HELLO_SPACE,
	SUBSEQUENT_ANGRY_HELLO_SPACE,
	INIT_NEUTRAL_HELLO_SPACE,
	SUBSEQUENT_NEUTRAL_HELLO_SPACE,
	INIT_FRIENDLY_HELLO_SPACE,
	SUBSEQUENT_FRIENDLY_HELLO_SPACE,
	INIT_ALLIED_HELLO_SPACE,
	SUBSEQUENT_ALLIED_HELLO_SPACE,
	give_info_space,
	HERES_SOME_INFO,
	we_sorry_space,
	APOLOGIZE_AT_HOMEWORLD,
	we_fight_again_space,
	OK_FIGHT_AGAIN_SPACE,
	bye_angry_space,
	GOODBYE_ANGRY_SPACE,
	look_weird,
	YOU_LOOK_WEIRD,
	no_look_really_weird,
	NO_YOU_LOOK_REALLY_WEIRD,
	come_in_peace,
	AGAINST_NATURE,
	prepare_to_die,
	ALWAYS_PREPARED,
	since_friendly_give_stuff,
	GIVE_ADVICE,
	whats_up_space_1,
	GENERAL_INFO_SPACE_1,
	bye_friendly_space,
	GOODBYE_FRIENDLY_SPACE,
	looking_for_a_few_good_squids,
	URQUAN_SLAVES,
	why_slaves,
	UMGAH_TRICK,
	tell_us_about_you,
	ABOUT_US,
	what_you_really_want,
	WANT_THIS,
	how_about_alliance,
	SURE,
	part_in_peace,
	KEEP_IT_SECRET,
	heard_youre_cowards,
	DARN_TOOTIN,
	wanna_fight,
	YES_WE_DO,
	so_lets_fight,
	OK_LETS_FIGHT,
	so_lets_fight_already,
	DONT_REALLY_WANT_TO_FIGHT,
	attack_you_now,
	YIPES,
	whats_up_space_2,
	GENERAL_INFO_SPACE_2,
	give_us_info_from_space,
	GET_INFO_FROM_SPATHIWA,
	give_us_resources_space,
	GET_RESOURCES_FROM_SPATHIWA,
	what_do_for_fun,
	DO_THIS_FOR_FUN,
	bye_ally_space,
	GOODBYE_ALLY_SPACE,
	OK_WE_FIGHT_AT_PLUTO,
};

#endif /* _STRINGS_H */
