#define YEHAT_ICON_MASK_PMAP_ANIM 0x00200002L
#define YEHAT_MICON_MASK_PMAP_ANIM 0x00200102L
#define YEHAT_BIG_MASK_PMAP_ANIM 0x00400202L
#define YEHAT_MED_MASK_PMAP_ANIM 0x00400302L
#define YEHAT_SML_MASK_PMAP_ANIM 0x00400402L
#define YEHAT_CANNON_BIG_MASK_PMAP_ANIM 0x00400502L
#define YEHAT_CANNON_MED_MASK_PMAP_ANIM 0x00400602L
#define YEHAT_CANNON_SML_MASK_PMAP_ANIM 0x00400702L
#define SHIELD_BIG_MASK_ANIM 0x00400802L
#define SHIELD_MED_MASK_ANIM 0x00400902L
#define SHIELD_SML_MASK_ANIM 0x00400a02L
#define YEHAT_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


