#define SAMATRA_BIG_MASK_ANIM 0x00400002L
#define SAMATRA_MED_MASK_PMAP_ANIM 0x00400102L
#define SAMATRA_SML_MASK_PMAP_ANIM 0x00400202L
#define SENTINEL_BIG_MASK_ANIM 0x00400302L
#define SENTINEL_MED_MASK_PMAP_ANIM 0x00400402L
#define SENTINEL_SML_MASK_PMAP_ANIM 0x00400502L
#define GENERATOR_BIG_MASK_ANIM 0x00400602L
#define GENERATOR_MED_MASK_PMAP_ANIM 0x00400702L
#define GENERATOR_SML_MASK_PMAP_ANIM 0x00400802L
#define SAMATRA_CAPTAIN_MASK_PMAP_ANIM 0x00600902L


