#define UMGAH_ICON_MASK_PMAP_ANIM 0x00200002L
#define UMGAH_MICON_MASK_PMAP_ANIM 0x00200102L
#define UMGAH_BIG_MASK_PMAP_ANIM 0x00400202L
#define UMGAH_MED_MASK_PMAP_ANIM 0x00400302L
#define UMGAH_SML_MASK_PMAP_ANIM 0x00400402L
#define SPRITZ_MASK_PMAP_ANIM 0x00400502L
#define CONE_BIG_MASK_ANIM 0x00400602L
#define CONE_MED_MASK_ANIM 0x00400702L
#define CONE_SML_MASK_ANIM 0x00400802L
#define UMGAH_CAPTAIN_MASK_PMAP_ANIM 0x00400902L


