#define DRUUGE_ICON_MASK_PMAP_ANIM 0x00200002L
#define DRUUGE_MICON_MASK_PMAP_ANIM 0x00200102L
#define DRUUGE_BIG_MASK_PMAP_ANIM 0x00400202L
#define DRUUGE_MED_MASK_PMAP_ANIM 0x00400302L
#define DRUUGE_SML_MASK_PMAP_ANIM 0x00400402L
#define DRUUGE_CANNON_BIG_MASK_PMAP_ANIM 0x00400502L
#define DRUUGE_CANNON_MED_MASK_PMAP_ANIM 0x00400602L
#define DRUUGE_CANNON_SML_MASK_PMAP_ANIM 0x00400702L
#define DRUUGE_CAPT_MASK_PMAP_ANIM 0x00400802L


