//Copyright Paul Reiche, Fred Ford. 1992-2002
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HELLO_SAMATRA,
	SENSE_EVIL,
	INIT_URQUAN_WAKE_UP,
	where_am_i,
	YOU_ARE_HERE,
	why_does_my_head_hurt,
	HURTS_BECAUSE,
	what_about_2_weeks,
	ABOUT_2_WEEKS,
	compulsion,
	WHAT_COMPULSION,
	wascally_little_guy,
	WHAT_IT_LOOK_LIKE,
	terran_amphibian,
	talking_pet_on_steroids,
	BAD_NEWS,
	turd_and_toad,
	WHAT_IS_TURD_AND_TOAD,
	alien_mind_control,
	WHAT_FELT_LIKE,
	possessed_by_devil,
	STUPID_DEVIL,
	falling_asleep,
	someone_else_controlled,
	SOUNDS_FAMILIAR,
	before_coffee,
	EXPLAIN,
	why_explain,
	MUST_EXPLAIN,
	bye_init_hypno,
	GOODBYE_AND_DIE_INIT_HYPNO,
	SUBSEQUENT_URQUAN_WAKE_UP,
	uh_oh,
	NO_UH_OH,
	stop_meeting,
	NO_STOP_MEETING,
	bye_sub_hypno,
	GOODBYE_AND_DIE_SUB_HYPNO,
	CAUGHT_YA,
	INIT_FLEE_HUMAN,
	SUBSEQUENT_FLEE_HUMAN,
	why_flee,
	FLEE_BECAUSE,
	what_happens_now,
	HAPPENS_NOW,
	what_about_you,
	ABOUT_US,
	bye_wars_over,
	GOODBYE_WARS_OVER,
	SEND_MESSAGE,
	INIT_HELLO,
	SUBSEQUENT_HELLO_1,
	SUBSEQUENT_HELLO_2,
	SUBSEQUENT_HELLO_3,
	SUBSEQUENT_HELLO_4,
	you_must_surrender,
	NOPE,
	i_surrender,
	DISOBEDIENCE_PUNISHED,
	i_wont_surrender,
	BAD_CHOICE,
	i_will_surrender,
	GOOD_CHOICE,
	key_phrase,
	URQUAN_STORY,
	like_to_leave,
	INDEPENDENCE_IS_BAD,
	whats_up_1,
	GENERAL_INFO_1,
	whats_up_2,
	GENERAL_INFO_2,
	whats_up_3,
	GENERAL_INFO_3,
	whats_up_4,
	GENERAL_INFO_4,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
