#define MELNORME_ICON_MASK_PMAP_ANIM 0x00200002L
#define MELNORME_MICON_MASK_PMAP_ANIM 0x00200102L
#define MELNORME_BIG_MASK_PMAP_ANIM 0x00400202L
#define MELNORME_MED_MASK_PMAP_ANIM 0x00400302L
#define MELNORME_SML_MASK_PMAP_ANIM 0x00400402L
#define PUMPUP_BIG_MASK_PMAP_ANIM 0x00400502L
#define PUMPUP_MED_MASK_PMAP_ANIM 0x00400602L
#define PUMPUP_SML_MASK_PMAP_ANIM 0x00400702L
#define CONFUSE_BIG_MASK_PMAP_ANIM 0x00400802L
#define CONFUSE_MED_MASK_PMAP_ANIM 0x00400902L
#define CONFUSE_SML_MASK_PMAP_ANIM 0x00400a02L
#define MELNORME_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


