#define TITLE_ANIM 0x00400002L
#define STATUS_MASK_PMAP_ANIM 0x03800102L
#define ACTIVITY_ANIM 0x03800202L
#define PLAYMENU_ANIM 0x03a00302L
#define FLAGSTAT_MASK_PMAP_ANIM 0x03e00402L
#define MISCDATA_MASK_PMAP_ANIM 0x03e00502L
#define STAR_MASK_PMAP_ANIM 0x04400602L
#define ASTEROID_BIG_MASK_PMAP_ANIM 0x04400702L
#define ASTEROID_MED_MASK_PMAP_ANIM 0x04400802L
#define ASTEROID_SML_MASK_PMAP_ANIM 0x04400902L
#define BLAST_BIG_MASK_PMAP_ANIM 0x04400a02L
#define BLAST_MED_MASK_PMAP_ANIM 0x04400b02L
#define BLAST_SML_MASK_PMAP_ANIM 0x04400c02L
#define BOOM_BIG_MASK_PMAP_ANIM 0x04400d02L
#define BOOM_MED_MASK_PMAP_ANIM 0x04400e02L
#define BOOM_SML_MASK_PMAP_ANIM 0x04400f02L
#define AMBIENT_MASK_PMAP_ANIM 0x04601002L
#define HYPERSTARS_MASK_PMAP_ANIM 0x04801102L
#define ARISPACE_MASK_PMAP_ANIM 0x04a01202L
#define IPBKGND_MASK_PMAP_ANIM 0x05001302L
#define SISIP_MASK_PMAP_ANIM 0x05201402L
#define ORBPLAN_MASK_PMAP_ANIM 0x05201502L
#define SUN_MASK_PMAP_ANIM 0x05201602L
#define LANDER_MASK_PMAP_ANIM 0x05401702L
#define QUAKE_MASK_PMAP_ANIM 0x05401802L
#define LIGHTNING_MASK_ANIM 0x05401902L
#define LAVA_MASK_PMAP_ANIM 0x05401a02L
#define LANDER_SHIELD_MASK_ANIM 0x05401b02L
#define LANDER_LAUNCH_MASK_PMAP_ANIM 0x05401c02L
#define LANDER_RETURN_MASK_PMAP_ANIM 0x05401d02L
#define ORBIT_VIEW_ANIM 0x05601e02L
#define EARTH_MASK_ANIM 0x05801f02L
#define CANNISTER_MASK_PMAP_ANIM 0x05a02002L
#define LIFE00_MASK_PMAP_ANIM 0x05c02102L
#define LIFE01_MASK_PMAP_ANIM 0x05e02202L
#define LIFE02_MASK_PMAP_ANIM 0x06002302L
#define LIFE03_MASK_PMAP_ANIM 0x06202402L
#define LIFE04_MASK_PMAP_ANIM 0x06402502L
#define LIFE05_MASK_PMAP_ANIM 0x06602602L
#define LIFE06_MASK_PMAP_ANIM 0x06802702L
#define LIFE07_MASK_PMAP_ANIM 0x06a02802L
#define LIFE08_MASK_PMAP_ANIM 0x06c02902L
#define LIFE09_MASK_PMAP_ANIM 0x06e02a02L
#define LIFE10_MASK_PMAP_ANIM 0x07002b02L
#define LIFE11_MASK_PMAP_ANIM 0x07202c02L
#define LIFE12_MASK_PMAP_ANIM 0x07402d02L
#define LIFE13_MASK_PMAP_ANIM 0x07602e02L
#define LIFE14_MASK_PMAP_ANIM 0x07802f02L
#define LIFE15_MASK_PMAP_ANIM 0x07a03002L
#define LIFE16_MASK_PMAP_ANIM 0x07c03102L
#define LIFE17_MASK_PMAP_ANIM 0x07e03202L
#define LIFE18_MASK_PMAP_ANIM 0x08003302L
#define LIFE19_MASK_PMAP_ANIM 0x08203402L
#define LIFE20_MASK_PMAP_ANIM 0x08403502L
#define LIFE21_MASK_PMAP_ANIM 0x08603602L
#define LIFE22_MASK_PMAP_ANIM 0x08803702L
#define LIFE23_MASK_PMAP_ANIM 0x08a03802L
#define LIFE24_MASK_PMAP_ANIM 0x08c03902L
#define LIFE25_MASK_PMAP_ANIM 0x08e03a02L
#define MOONBASE_MASK_PMAP_ANIM 0x11603b02L
#define MAIDENS_MASK_PMAP_ANIM 0x11803c02L
#define AQUA_MASK_PMAP_ANIM 0x11c03d02L
#define BURV_BCS_MASK_PMAP_ANIM 0x11e03e02L
#define TAALO_DEVICE_MASK_PMAP_ANIM 0x12003f02L
#define SUN_DEVICE_MASK_PMAP_ANIM 0x12204002L
#define VAULT_MASK_PMAP_ANIM 0x12404102L
#define WRECK_MASK_PMAP_ANIM 0x12604202L
#define BOMB_MASK_PMAP_ANIM 0x12804302L
#define EGG_CASE_MASK_PMAP_ANIM 0x12c04402L
#define SPAPLUTO_MASK_PMAP_ANIM 0x12e04502L
#define RUINS_MASK_PMAP_ANIM 0x13804602L
#define UMGAH_BCS_MASK_PMAP_ANIM 0x13c04702L
#define MELEE_SCREEN_PMAP_ANIM 0x14204802L
#define MELEE_PICK_MASK_PMAP_ANIM 0x14404902L
#define SEGUE_PMAP_ANIM 0x14604a02L
#define SC2_PICK_PMAP_ANIM 0x14804b02L
#define RESTART_PMAP_ANIM 0x18204c02L
#define MODULES_PMAP_ANIM 0x18404d02L
#define SISMODS_MASK_PMAP_ANIM 0x18604e02L
#define OUTFIT_PMAP_ANIM 0x18604f02L
#define SISBLU_MASK_ANIM 0x18a05002L
#define SHIPYARD_PMAP_ANIM 0x18a05102L
#define STARBASE_ANIM 0x18e05202L
#define SAMATRA_BIG_MASK_PMAP_ANIM 0x19005302L
#define SHIELDED_BIG_MASK_PMAP_ANIM 0x19205402L
#define SHIELDED_MED_MASK_PMAP_ANIM 0x19205502L
#define SHIELDED_SML_MASK_PMAP_ANIM 0x19205602L
#define PLANET00_BIG_MASK_PMAP_ANIM 0x19405702L
#define PLANET00_MED_MASK_PMAP_ANIM 0x19405802L
#define PLANET00_SML_MASK_PMAP_ANIM 0x19405902L
#define PLANET01_BIG_MASK_PMAP_ANIM 0x19605a02L
#define PLANET01_MED_MASK_PMAP_ANIM 0x19605b02L
#define PLANET01_SML_MASK_PMAP_ANIM 0x19605c02L
#define PLANET02_BIG_MASK_PMAP_ANIM 0x19805d02L
#define PLANET02_MED_MASK_PMAP_ANIM 0x19805e02L
#define PLANET02_SML_MASK_PMAP_ANIM 0x19805f02L
#define PLANET03_BIG_MASK_PMAP_ANIM 0x19a06002L
#define PLANET03_MED_MASK_PMAP_ANIM 0x19a06102L
#define PLANET03_SML_MASK_PMAP_ANIM 0x19a06202L
#define PLANET04_BIG_MASK_PMAP_ANIM 0x19c06302L
#define PLANET04_MED_MASK_PMAP_ANIM 0x19c06402L
#define PLANET04_SML_MASK_PMAP_ANIM 0x19c06502L
#define PLANET05_BIG_MASK_PMAP_ANIM 0x19e06602L
#define PLANET05_MED_MASK_PMAP_ANIM 0x19e06702L
#define PLANET05_SML_MASK_PMAP_ANIM 0x19e06802L
#define PLANET06_BIG_MASK_PMAP_ANIM 0x1a006902L
#define PLANET06_MED_MASK_PMAP_ANIM 0x1a006a02L
#define PLANET06_SML_MASK_PMAP_ANIM 0x1a006b02L
#define PLANET07_BIG_MASK_PMAP_ANIM 0x1a206c02L
#define PLANET07_MED_MASK_PMAP_ANIM 0x1a206d02L
#define PLANET07_SML_MASK_PMAP_ANIM 0x1a206e02L
#define PLANET08_BIG_MASK_PMAP_ANIM 0x1a406f02L
#define PLANET08_MED_MASK_PMAP_ANIM 0x1a407002L
#define PLANET08_SML_MASK_PMAP_ANIM 0x1a407102L
#define PLANET09_BIG_MASK_PMAP_ANIM 0x1a607202L
#define PLANET09_MED_MASK_PMAP_ANIM 0x1a607302L
#define PLANET09_SML_MASK_PMAP_ANIM 0x1a607402L
#define PLANET10_BIG_MASK_PMAP_ANIM 0x1a807502L
#define PLANET10_MED_MASK_PMAP_ANIM 0x1a807602L
#define PLANET10_SML_MASK_PMAP_ANIM 0x1a807702L
#define PLANET11_BIG_MASK_PMAP_ANIM 0x1aa07802L
#define PLANET11_MED_MASK_PMAP_ANIM 0x1aa07902L
#define PLANET11_SML_MASK_PMAP_ANIM 0x1aa07a02L
#define PLANET12_BIG_MASK_PMAP_ANIM 0x1ac07b02L
#define PLANET12_MED_MASK_PMAP_ANIM 0x1ac07c02L
#define PLANET12_SML_MASK_PMAP_ANIM 0x1ac07d02L
#define PLANET13_BIG_MASK_PMAP_ANIM 0x1ae07e02L
#define PLANET13_MED_MASK_PMAP_ANIM 0x1ae07f02L
#define PLANET13_SML_MASK_PMAP_ANIM 0x1ae08002L
#define PLANET14_BIG_MASK_PMAP_ANIM 0x1b008102L
#define PLANET14_MED_MASK_PMAP_ANIM 0x1b008202L
#define PLANET14_SML_MASK_PMAP_ANIM 0x1b008302L
#define PLANET15_BIG_MASK_PMAP_ANIM 0x1b208402L
#define PLANET15_MED_MASK_PMAP_ANIM 0x1b208502L
#define PLANET15_SML_MASK_PMAP_ANIM 0x1b208602L
#define PLANET16_BIG_MASK_PMAP_ANIM 0x1b408702L
#define PLANET16_MED_MASK_PMAP_ANIM 0x1b408802L
#define PLANET16_SML_MASK_PMAP_ANIM 0x1b408902L
#define PLANET17_BIG_MASK_PMAP_ANIM 0x1b608a02L
#define PLANET17_MED_MASK_PMAP_ANIM 0x1b608b02L
#define PLANET17_SML_MASK_PMAP_ANIM 0x1b608c02L
#define PLANET18_BIG_MASK_PMAP_ANIM 0x1b808d02L
#define PLANET18_MED_MASK_PMAP_ANIM 0x1b808e02L
#define PLANET18_SML_MASK_PMAP_ANIM 0x1b808f02L
#define PLANET19_BIG_MASK_PMAP_ANIM 0x1ba09002L
#define PLANET19_MED_MASK_PMAP_ANIM 0x1ba09102L
#define PLANET19_SML_MASK_PMAP_ANIM 0x1ba09202L
#define PLANET20_BIG_MASK_PMAP_ANIM 0x1bc09302L
#define PLANET20_MED_MASK_PMAP_ANIM 0x1bc09402L
#define PLANET20_SML_MASK_PMAP_ANIM 0x1bc09502L
#define PLANET21_BIG_MASK_PMAP_ANIM 0x1be09602L
#define PLANET21_MED_MASK_PMAP_ANIM 0x1be09702L
#define PLANET21_SML_MASK_PMAP_ANIM 0x1be09802L
#define PLANET22_BIG_MASK_PMAP_ANIM 0x1c009902L
#define PLANET22_MED_MASK_PMAP_ANIM 0x1c009a02L
#define PLANET22_SML_MASK_PMAP_ANIM 0x1c009b02L
#define PLANET23_BIG_MASK_PMAP_ANIM 0x1c209c02L
#define PLANET23_MED_MASK_PMAP_ANIM 0x1c209d02L
#define PLANET23_SML_MASK_PMAP_ANIM 0x1c209e02L
#define PLANET24_BIG_MASK_PMAP_ANIM 0x1c409f02L
#define PLANET24_MED_MASK_PMAP_ANIM 0x1c40a002L
#define PLANET24_SML_MASK_PMAP_ANIM 0x1c40a102L
#define PLANET25_BIG_MASK_PMAP_ANIM 0x1c60a202L
#define PLANET25_MED_MASK_PMAP_ANIM 0x1c60a302L
#define PLANET25_SML_MASK_PMAP_ANIM 0x1c60a402L
#define PLANET26_BIG_MASK_PMAP_ANIM 0x1c80a502L
#define PLANET26_MED_MASK_PMAP_ANIM 0x1c80a602L
#define PLANET26_SML_MASK_PMAP_ANIM 0x1c80a702L
#define PLANET27_BIG_MASK_PMAP_ANIM 0x1ca0a802L
#define PLANET27_MED_MASK_PMAP_ANIM 0x1ca0a902L
#define PLANET27_SML_MASK_PMAP_ANIM 0x1ca0aa02L
#define PLANET28_BIG_MASK_PMAP_ANIM 0x1cc0ab02L
#define PLANET28_MED_MASK_PMAP_ANIM 0x1cc0ac02L
#define PLANET28_SML_MASK_PMAP_ANIM 0x1cc0ad02L
#define PLANET29_BIG_MASK_PMAP_ANIM 0x1ce0ae02L
#define PLANET29_MED_MASK_PMAP_ANIM 0x1ce0af02L
#define PLANET29_SML_MASK_PMAP_ANIM 0x1ce0b002L
#define PLANET30_BIG_MASK_PMAP_ANIM 0x1d00b102L
#define PLANET30_MED_MASK_PMAP_ANIM 0x1d00b202L
#define PLANET30_SML_MASK_PMAP_ANIM 0x1d00b302L
#define PLANET31_BIG_MASK_PMAP_ANIM 0x1d20b402L
#define PLANET31_MED_MASK_PMAP_ANIM 0x1d20b502L
#define PLANET31_SML_MASK_PMAP_ANIM 0x1d20b602L
#define PLANET32_BIG_MASK_PMAP_ANIM 0x1d40b702L
#define PLANET32_MED_MASK_PMAP_ANIM 0x1d40b802L
#define PLANET32_SML_MASK_PMAP_ANIM 0x1d40b902L
#define PLANET33_BIG_MASK_PMAP_ANIM 0x1d60ba02L
#define PLANET33_MED_MASK_PMAP_ANIM 0x1d60bb02L
#define PLANET33_SML_MASK_PMAP_ANIM 0x1d60bc02L
#define PLANET34_BIG_MASK_PMAP_ANIM 0x1d80bd02L
#define PLANET34_MED_MASK_PMAP_ANIM 0x1d80be02L
#define PLANET34_SML_MASK_PMAP_ANIM 0x1d80bf02L
#define PLANET35_BIG_MASK_PMAP_ANIM 0x1da0c002L
#define PLANET35_MED_MASK_PMAP_ANIM 0x1da0c102L
#define PLANET35_SML_MASK_PMAP_ANIM 0x1da0c202L
#define PLANET36_BIG_MASK_PMAP_ANIM 0x1dc0c302L
#define PLANET36_MED_MASK_PMAP_ANIM 0x1dc0c402L
#define PLANET36_SML_MASK_PMAP_ANIM 0x1dc0c502L
#define PLANET37_BIG_MASK_PMAP_ANIM 0x1de0c602L
#define PLANET37_MED_MASK_PMAP_ANIM 0x1de0c702L
#define PLANET37_SML_MASK_PMAP_ANIM 0x1de0c802L
#define PLANET38_BIG_MASK_PMAP_ANIM 0x1e00c902L
#define PLANET38_MED_MASK_PMAP_ANIM 0x1e00ca02L
#define PLANET38_SML_MASK_PMAP_ANIM 0x1e00cb02L
#define PLANET39_BIG_MASK_PMAP_ANIM 0x1e20cc02L
#define PLANET39_MED_MASK_PMAP_ANIM 0x1e20cd02L
#define PLANET39_SML_MASK_PMAP_ANIM 0x1e20ce02L
#define PLANET40_BIG_MASK_PMAP_ANIM 0x1e40cf02L
#define PLANET40_MED_MASK_PMAP_ANIM 0x1e40d002L
#define PLANET40_SML_MASK_PMAP_ANIM 0x1e40d102L
#define PLANET41_BIG_MASK_PMAP_ANIM 0x1e60d202L
#define PLANET41_MED_MASK_PMAP_ANIM 0x1e60d302L
#define PLANET41_SML_MASK_PMAP_ANIM 0x1e60d402L
#define PLANET42_BIG_MASK_PMAP_ANIM 0x1e80d502L
#define PLANET42_MED_MASK_PMAP_ANIM 0x1e80d602L
#define PLANET42_SML_MASK_PMAP_ANIM 0x1e80d702L
#define PLANET43_BIG_MASK_PMAP_ANIM 0x1ea0d802L
#define PLANET43_MED_MASK_PMAP_ANIM 0x1ea0d902L
#define PLANET43_SML_MASK_PMAP_ANIM 0x1ea0da02L
#define PLANET44_BIG_MASK_PMAP_ANIM 0x1ec0db02L
#define PLANET44_MED_MASK_PMAP_ANIM 0x1ec0dc02L
#define PLANET44_SML_MASK_PMAP_ANIM 0x1ec0dd02L
#define PLANET45_BIG_MASK_PMAP_ANIM 0x1ee0de02L
#define PLANET45_MED_MASK_PMAP_ANIM 0x1ee0df02L
#define PLANET45_SML_MASK_PMAP_ANIM 0x1ee0e002L
#define PLANET46_BIG_MASK_PMAP_ANIM 0x1f00e102L
#define PLANET46_MED_MASK_PMAP_ANIM 0x1f00e202L
#define PLANET46_SML_MASK_PMAP_ANIM 0x1f00e302L
#define PLANET47_BIG_MASK_PMAP_ANIM 0x1f20e402L
#define PLANET47_MED_MASK_PMAP_ANIM 0x1f20e502L
#define PLANET47_SML_MASK_PMAP_ANIM 0x1f20e602L
#define PLANET48_BIG_MASK_PMAP_ANIM 0x1f40e702L
#define PLANET48_MED_MASK_PMAP_ANIM 0x1f40e802L
#define PLANET48_SML_MASK_PMAP_ANIM 0x1f40e902L
#define PLANET49_BIG_MASK_PMAP_ANIM 0x1f60ea02L
#define PLANET49_MED_MASK_PMAP_ANIM 0x1f60eb02L
#define PLANET49_SML_MASK_PMAP_ANIM 0x1f60ec02L
#define PLANET50_BIG_MASK_PMAP_ANIM 0x1f80ed02L
#define PLANET50_MED_MASK_PMAP_ANIM 0x1f80ee02L
#define PLANET50_SML_MASK_PMAP_ANIM 0x1f80ef02L
#define PLANET51_BIG_MASK_PMAP_ANIM 0x1fa0f002L
#define PLANET51_MED_MASK_PMAP_ANIM 0x1fa0f102L
#define PLANET51_SML_MASK_PMAP_ANIM 0x1fa0f202L
#define PLANET52_BIG_MASK_PMAP_ANIM 0x1fc0f302L
#define PLANET52_MED_MASK_PMAP_ANIM 0x1fc0f402L
#define PLANET52_SML_MASK_PMAP_ANIM 0x1fc0f502L
#define PLANET53_BIG_MASK_PMAP_ANIM 0x1fe0f602L
#define PLANET53_MED_MASK_PMAP_ANIM 0x1fe0f702L
#define PLANET53_SML_MASK_PMAP_ANIM 0x1fe0f802L
#define PLANET54_BIG_MASK_PMAP_ANIM 0x2000f902L
#define PLANET54_MED_MASK_PMAP_ANIM 0x2000fa02L
#define PLANET54_SML_MASK_PMAP_ANIM 0x2000fb02L
#define PLANET55_BIG_MASK_PMAP_ANIM 0x2020fc02L
#define PLANET55_MED_MASK_PMAP_ANIM 0x2020fd02L
#define PLANET55_SML_MASK_PMAP_ANIM 0x2020fe02L
#define PLANET56_BIG_MASK_PMAP_ANIM 0x2040ff02L
#define PLANET56_MED_MASK_PMAP_ANIM 0x20410002L
#define PLANET56_SML_MASK_PMAP_ANIM 0x20410102L
#define PLANET57_BIG_MASK_PMAP_ANIM 0x20610202L
#define PLANET57_MED_MASK_PMAP_ANIM 0x20610302L
#define PLANET57_SML_MASK_PMAP_ANIM 0x20610402L
#define PLANET58_BIG_MASK_PMAP_ANIM 0x20810502L
#define PLANET58_MED_MASK_PMAP_ANIM 0x20810602L
#define PLANET58_SML_MASK_PMAP_ANIM 0x20810702L
#define CREDITS_BACK_ANIM 0x20c10802L

#define SISSKEL_MASK_PMAP_ANIM 0x23611c02L
#define ORBENTER_PMAP_ANIM 0x23811d02L
#define MENUBKG_PMAP_ANIM 0x23a11e02L
#define FONTGRAD_PMAP_ANIM 0x03e11f02L
#define LANDER_FONTEFF_PMAP_ANIM 0x11412002L

