//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	WHY_YOU_HERE_1,
	WHY_YOU_HERE_2,
	WHY_YOU_HERE_3,
	WHY_YOU_HERE_4,
	find_out_whats_up,
	HYBRID_PROCESS,
	need_help,
	CANT_HELP,
	why_no_help,
	LONG_TIME,
	what_if_more_energy,
	DANGER_TO_US,
	need_advice,
	WHAT_ADVICE,
	how_defeat_urquan,
	DEFEAT_LIKE_SO,
	what_about_tpet,
	SCARY_BUT_USEFUL,
	what_about_bomb,
	ABOUT_BOMB,
	what_about_sun_device,
	ABOUT_SUN_DEVICE,
	what_about_samatra,
	ABOUT_SAMATRA,
	enough_advice,
	OK_ENOUGH_ADVICE,
	bye_shielded,
	GOODBYE_SHIELDED,
	WE_ARE_FREE,
	WHO_ARE_YOU,
	i_am_captain0,
	i_am_captain1,
	i_am_captain2,
	i_am_savior,
	i_am_silly,
	WHY_HAVE_YOU_FREED_US,
	serious_1,
	serious_2,
	silly,
	WILL_HELP_ANALYZE_LOGS,
	YOU_KNOW_SAMATRA,
	DONT_KNOW_ABOUT_SAMATRA,
	NEED_DISTRACTION,
	HAVE_TALKING_PET,
	NEED_WEAPON,
	HAVE_BOMB,
	RETURN_WHEN_READY,
	YOU_ARE_READY,
	further_assistance,
	NO_FURTHER_ASSISTANCE,
	tech_help,
	USE_OUR_SHIPS_BEFORE,
	where_weapon,
	PRECURSOR_WEAPON,
	where_distraction,
	PSYCHIC_WEAPONRY,
	what_now,
	WE_WILL_IMPROVE_BOMB,
	MODIFY_VESSEL,
	wont_hurt_my_ship,
	WILL_DESTROY_IT,
	bummer_about_my_ship,
	DEAD_SILENCE,
	other_assistance,
	USE_OUR_SHIPS_AFTER,
	proceed,
	TAKE_2_WEEKS,
	HELLO_AFTER_BOMB_1,
	HELLO_AFTER_BOMB_2,
	whats_up_after_bomb,
	GENERAL_INFO_AFTER_BOMB_1,
	GENERAL_INFO_AFTER_BOMB_2,
	what_do_after_bomb,
	DO_AFTER_BOMB,
	bye_after_bomb,
	GOODBYE_AFTER_BOMB,
	bye,
	GOODBYE,
};

#endif /* _STRINGS_H */
