//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HOSTILE_SPACE_HELLO_1,
	HOSTILE_SPACE_HELLO_2,
	HOSTILE_SPACE_HELLO_3,
	HOSTILE_SPACE_HELLO_4,
	HOSTILE_HOMEWORLD_HELLO_1,
	HOSTILE_HOMEWORLD_HELLO_2,
	HOSTILE_HOMEWORLD_HELLO_3,
	HOSTILE_HOMEWORLD_HELLO_4,
	whats_up_hostile_1,
	whats_up_hostile_2,
	GENERAL_INFO_HOSTILE_1,
	GENERAL_INFO_HOSTILE_2,
	GENERAL_INFO_HOSTILE_3,
	GENERAL_INFO_HOSTILE_4,
	what_about_you_1,
	ABOUT_US_1,
	what_about_you_2,
	ABOUT_US_2,
	what_about_urquan_1,
	ABOUT_URQUAN_1,
	what_about_urquan_2,
	ABOUT_URQUAN_2,
	got_idea,
	GOOD_IDEA,
	WE_GO_TO_IMPRESS_URQUAN_1,
	WE_GO_TO_IMPRESS_URQUAN_2,
	WE_IMPRESSING_URQUAN_1,
	WE_IMPRESSING_URQUAN_2,
	WE_IMPRESSED_URQUAN_1,
	WE_IMPRESSED_URQUAN_2,
	HOSTILE_HELIX_HELLO_1,
	HOSTILE_HELIX_HELLO_2,
	submit_1,
	NO_SUBMIT_1,
	submit_2,
	NO_SUBMIT_2,
	be_friends_1,
	NO_FRIENDS_1,
	be_friends_2,
	NO_FRIENDS_2,
	how_impressed_urquan_1,
	IMPRESSED_LIKE_SO_1,
	how_impressed_urquan_2,
	IMPRESSED_LIKE_SO_2,
	bye_hostile_1,
	GOODBYE_HOSTILE_1,
	bye_hostile_2,
	GOODBYE_HOSTILE_2,
	why_you_here_hostile,
	NONE_OF_YOUR_CONCERN,
	demand_to_land,
	NO_DEMAND,
	what_about_this_world,
	BLUE_HELIX,
	whats_helix_hostile,
	HELIX_IS_HOSTILE,
	i_need_to_land_lie,
	CAUGHT_LIE,
	bye_hostile_helix,
	GOODBYE_HOSTILE_HELIX,
	DIE_THIEF_1,
	DIE_THIEF_2,
	AMAZING_PERFORMANCE,
	IMPRESSIVE_PERFORMANCE,
	ADEQUATE_PERFORMANCE,
	HELLO_POLITE_1,
	HELLO_POLITE_2,
	HELLO_POLITE_3,
	HELLO_POLITE_4,
	HELLO_RHYME_1,
	HELLO_RHYME_2,
	HELLO_RHYME_3,
	HELLO_RHYME_4,
	HELLO_PIG_LATIN_1,
	HELLO_PIG_LATIN_2,
	HELLO_PIG_LATIN_3,
	HELLO_PIG_LATIN_4,
	HELLO_LIKE_YOU_1,
	HELLO_LIKE_YOU_2,
	HELLO_LIKE_YOU_3,
	HELLO_LIKE_YOU_4,
	WELCOME_SPACE0,
	WELCOME_SPACE1,
	WELCOME_HOMEWORLD0,
	WELCOME_HOMEWORLD1,
	WELCOME_HELIX0,
	WELCOME_HELIX1,
	why_you_here_ally,
	GUARDING_HELIX_ALLY,
	whats_helix_ally,
	HELIX_IS_ALLY,
	may_i_land,
	SURE_LAND,
	whats_up_ally,
	GENERAL_INFO_ALLY_1,
	GENERAL_INFO_ALLY_2,
	GENERAL_INFO_ALLY_3,
	GENERAL_INFO_ALLY_4,
	HOW_SHOULD_WE_ACT,
	friendly,
	OK_FRIENDLY,
	wacky,
	OK_WACKY,
	just_like_us,
	OK_JUST_LIKE_YOU,
	WORK_TO_DO,
	contemplative,
	OK_CONTEMPLATIVE,
	how_goes_culture,
	CONTEMP_GOES_1,
	CONTEMP_GOES_2,
	FRIENDLY_GOES_1,
	FRIENDLY_GOES_2,
	WACKY_GOES_1,
	WACKY_GOES_2,
	LIKE_YOU_GOES_1,
	LIKE_YOU_GOES_2,
	bye_ally,
	GOODBYE_ALLY_1,
	GOODBYE_ALLY_2,
	GOODBYE_ALLY_3,
	GOODBYE_ALLY_4,
	be_polite,
	OK_POLITE,
	speak_pig_latin,
	OK_PIG_LATIN,
	use_rhymes,
	OK_RHYMES,
	just_the_way_we_do,
	OK_WAY_YOU_DO,
	WHAT_NAME_FOR_CULTURE,
	alliance_name,
	OK_ALLIANCE_NAME,
	NAME_TAIL,
	you_decide,
	OK_CULTURE_20,
	fat,
	OK_FAT,
	the_slave_empire0,
	the_slave_empire1,
	OK_SLAVE,
	FAT_JERKS,
	CULTURE,
	SLAVE_EMPIRE,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	HAVING_FUN_WITH_ILWRATH_1,
	HAVING_FUN_WITH_ILWRATH_2,
	GO_AWAY_FIGHTING_ILWRATH_1,
	GO_AWAY_FIGHTING_ILWRATH_2,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
