//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	NEUTRAL_SPACE_HELLO_1,
	NEUTRAL_SPACE_HELLO_2,
	NEUTRAL_HOMEWORLD_HELLO_1,
	NEUTRAL_HOMEWORLD_HELLO_2,
	HOSTILE_SPACE_HELLO_1,
	HOSTILE_SPACE_HELLO_2,
	ALLIED_HOMEWORLD_HELLO_1,
	ALLIED_HOMEWORLD_HELLO_2,
	ALLIED_HOMEWORLD_HELLO_3,
	ALLIED_HOMEWORLD_HELLO_4,
	i_am0,
	i_am1,
	WE_ARE_SUPOX,
	my_ship0,
	my_ship1,
	OUR_SHIP,
	from_alliance0,
	from_alliance1,
	FROM_SUPOX,
	are_you_copying,
	YEAH_SORRY,
	why_copy,
	SYMBIOTS,
	tell_us_of_your_species,
	OUR_SPECIES,
	plants_arent_intelligent,
	PROVES_WERE_SPECIAL,
	anyone_around_here,
	UTWIG_NEARBY,
	what_relation_to_utwig,
	UTWIG_ALLIES,
	whats_wrong_with_utwig,
	BROKE_ULTRON,
	whats_ultron,
	TAKE_ULTRON,
	what_do_i_do_now,
	FIX_IT,
	thanks_now_we_eat_you,
	HIDEOUS_MONSTERS,
	got_fixed_ultron,
	GOOD_GIVE_TO_UTWIG,
	look_i_repaired_lots,
	ALMOST_THERE,
	look_i_slightly_repaired,
	GREAT_DO_MORE,
	where_get_repairs,
	ANCIENT_RHYME,
	bye_neutral,
	GOODBYE_NEUTRAL,
	ABOUT_BATTLE,
	HELLO_BEFORE_KOHRAH_SPACE_1,
	HELLO_BEFORE_KOHRAH_SPACE_2,
	HELLO_DURING_KOHRAH_SPACE_1,
	HELLO_DURING_KOHRAH_SPACE_2,
	HELLO_AFTER_KOHRAH_SPACE_1,
	HELLO_AFTER_KOHRAH_SPACE_2,
	whats_up_after_space,
	GENERAL_INFO_AFTER_SPACE_1,
	GENERAL_INFO_AFTER_SPACE_2,
	what_now_after_space,
	DO_THIS_AFTER_SPACE,
	bye_after_space,
	GOODBYE_AFTER_SPACE,
	whats_up_before_space,
	GENERAL_INFO_BEFORE_SPACE_1,
	GENERAL_INFO_BEFORE_SPACE_2,
	what_now_before_space,
	DO_THIS_BEFORE_SPACE,
	bye_before_space,
	GOODBYE_BEFORE_SPACE,
	how_went_war,
	how_goes_war,
	BATTLE_HAPPENS_1,
	BATTLE_HAPPENS_2,
	FLEET_ON_WAY,
	learn_new_info,
	NO_NEW_INFO,
	SAMATRA,
	what_now_homeworld,
	HOPE_KILL_EACH_OTHER,
	UP_TO_YOU,
	can_you_help,
	HOW_HELP,
	DONT_NEED,
	HAVE_4_SHIPS,
	give_info,
	GOOD_HINTS,
	how_is_ultron,
	ULTRON_IS_GREAT,
	bye_allied_homeworld,
	GOODBYE_ALLIED_HOMEWORLD,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
