//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	REBEL_HELLO_1,
	REBEL_HELLO_2,
	REBEL_HELLO_3,
	REBEL_HELLO_4,
	how_goes_revolution,
	REBEL_REVOLUTION_1,
	REBEL_REVOLUTION_2,
	REBEL_REVOLUTION_3,
	REBEL_REVOLUTION_4,
	any_ships,
	NO_ROOM,
	HAVE_ALL_SHIPS,
	HAVE_FEW_SHIPS,
	NO_SHIPS_YET,
	give_info_rebels,
	WHAT_INFO,
	what_about_royalty,
	ABOUT_ROYALTY,
	what_about_war,
	ABOUT_WAR,
	what_about_urquan,
	ABOUT_URQUAN,
	what_about_vux,
	ABOUT_VUX,
	what_about_clue,
	ABOUT_CLUE,
	enough_info,
	OK_ENOUGH_INFO,
	bye_rebel,
	GOODBYE_REBEL,
	YEHAT_CAVALRY,
	what_about_pkunk_rebel,
	PKUNK_ABSORBED_REBEL,
	HATE_PKUNK_REBEL,
};

#endif /* _STRINGS_H */
