#define ARILOU_SHIP_INDEX 0x00600007L
#define CHMMR_SHIP_INDEX 0x00800107L
#define EARTHLING_SHIP_INDEX 0x00a00207L
#define ORZ_SHIP_INDEX 0x00c00307L
#define PKUNK_SHIP_INDEX 0x00e00407L
#define SHOFIXTI_SHIP_INDEX 0x01000507L
#define SPATHI_SHIP_INDEX 0x01200607L
#define SUPOX_SHIP_INDEX 0x01400707L
#define THRADD_SHIP_INDEX 0x01600807L
#define UTWIG_SHIP_INDEX 0x01800907L
#define VUX_SHIP_INDEX 0x01a00a07L
#define YEHAT_SHIP_INDEX 0x01c00b07L
#define MELNORME_SHIP_INDEX 0x01e00c07L
#define DRUUGE_SHIP_INDEX 0x02000d07L
#define ILWRATH_SHIP_INDEX 0x02200e07L
#define MYCON_SHIP_INDEX 0x02400f07L
#define SLYLANDRO_SHIP_INDEX 0x02601007L
#define UMGAH_SHIP_INDEX 0x02801107L
#define URQUAN_SHIP_INDEX 0x02a01207L
#define ZOQFOTPIK_SHIP_INDEX 0x02c01307L
#define SYREEN_SHIP_INDEX 0x02e01407L
#define BLACKURQ_SHIP_INDEX 0x03001507L
#define ANDROSYN_SHIP_INDEX 0x03201607L
#define CHENJESU_SHIP_INDEX 0x03401707L
#define MMRNMHRM_SHIP_INDEX 0x03601807L
#define ARILOU_CONVERSATION 0x14a01907L
#define CHMMR_CONVERSATION 0x14c01a07L
#define COMMANDER_CONVERSATION 0x14e01b07L
#define ORZ_CONVERSATION 0x15001c07L
#define PKUNK_CONVERSATION 0x15201d07L
#define SHOFIXTI_CONVERSATION 0x15401e07L
#define SPATHI_CONVERSATION 0x15601f07L
#define SUPOX_CONVERSATION 0x15802007L
#define THRADD_CONVERSATION 0x15a02107L
#define UTWIG_CONVERSATION 0x15c02207L
#define VUX_CONVERSATION 0x15e02307L
#define YEHAT_CONVERSATION 0x16002407L
#define MELNORME_CONVERSATION 0x16202507L
#define DRUUGE_CONVERSATION 0x16402607L
#define ILWRATH_CONVERSATION 0x16602707L
#define MYCON_CONVERSATION 0x16802807L
#define SLYLANDRO_CONVERSATION 0x16a02907L
#define UMGAH_CONVERSATION 0x16c02a07L
#define URQUAN_CONVERSATION 0x16e02b07L
#define ZOQFOTPIK_CONVERSATION 0x17002c07L
#define SYREEN_CONVERSATION 0x17202d07L
#define BLACKURQ_CONVERSATION 0x17402e07L
#define TALKING_PET_CONVERSATION 0x17602f07L
#define SLYLANDRO_HOME_CONVERSATION 0x17803007L
#define SIS_RES_INDEX 0x17c03107L
#define SAMATRA_RES_INDEX 0x17e03207L
#define PROBE_RES_INDEX 0x18003307L


