#define URQUAN_ICON_MASK_PMAP_ANIM 0x00200002L
#define URQUAN_MICON_MASK_PMAP_ANIM 0x00200102L
#define URQUAN_BIG_MASK_PMAP_ANIM 0x00400202L
#define URQUAN_MED_MASK_PMAP_ANIM 0x00400302L
#define URQUAN_SML_MASK_PMAP_ANIM 0x00400402L
#define FUSION_BIG_MASK_PMAP_ANIM 0x00400502L
#define FUSION_MED_MASK_PMAP_ANIM 0x00400602L
#define FUSION_SML_MASK_PMAP_ANIM 0x00400702L
#define FIGHTER_BIG_MASK_PMAP_ANIM 0x00400802L
#define FIGHTER_MED_MASK_PMAP_ANIM 0x00400902L
#define FIGHTER_SML_MASK_PMAP_ANIM 0x00400a02L
#define URQUAN_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


