//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HWLD_PRE_ZOMBIE_HELLO_1,
	HWLD_PRE_ZOMBIE_HELLO_2,
	HWLD_PRE_ZOMBIE_HELLO_3,
	HWLD_PRE_ZOMBIE_HELLO_4,
	SPACE_PRE_ZOMBIE_HELLO_1,
	SPACE_PRE_ZOMBIE_HELLO_2,
	SPACE_PRE_ZOMBIE_HELLO_3,
	SPACE_PRE_ZOMBIE_HELLO_4,
	UNKNOWN_ZOMBIE_HELLO_1,
	UNKNOWN_ZOMBIE_HELLO_2,
	UNKNOWN_ZOMBIE_HELLO_3,
	UNKNOWN_ZOMBIE_HELLO_4,
	DESTROY_INTERFERER_1,
	DESTROY_INTERFERER_2,
	DESTROY_INTERFERER_3,
	DESTROY_INTERFERER_4,
	REVEALED_ZOMBIE_HELLO_1,
	REVEALED_ZOMBIE_HELLO_2,
	REVEALED_ZOMBIE_HELLO_3,
	REVEALED_ZOMBIE_HELLO_4,
	HOSTILE_HELLO_1,
	HOSTILE_HELLO_2,
	HOSTILE_HELLO_3,
	HOSTILE_HELLO_4,
	REWARD_AT_HOMEWORLD_1,
	REWARD_AT_HOMEWORLD_2,
	POST_ZOMBIE_HWLD_HELLO,
	owe_me_big_time,
	our_largesse,
	GIVE_LIFEDATA,
	THANKS,
	what_do_with_tpet,
	TRICK_URQUAN,
	any_jokes,
	SURE,
	what_before_tpet,
	TRKD_SPATHI_AND_ILWRATH,
	where_caster,
	SPATHI_TOOK_THEM,
	so_what_for_now,
	DO_THIS_NOW,
	bye_post_zombie,
	FUNNY_IDEA,
	whats_up_pre_zombie,
	GENERAL_INFO_PRE_ZOMBIE,
	evil_blobbies_give_up,
	NOT_EVIL_BLOBBIES,
	evil_blobbies_must_die,
	OH_NO_WE_WONT,
	can_we_be_friends,
	SURE_FRIENDS,
	want_to_defeat_urquan,
	FINE_BY_US,
	bye_pre_zombie,
	GOODBYE_PRE_ZOMBIE,
	threat,
	NO_THREAT,
	whats_up_zombies,
	GENERAL_INFO_ZOMBIE,
	how_goes_tpet,
	WHAT_TPET,
	you_told_us,
	SADLY_IT_DIED,
	dont_believe,
	THEN_DIE,
	bye_unknown,
	GOODBYE_UNKNOWN,
	evil_blobbies,
	YES_VERY_EVIL,
	give_up_or_die,
	NOT_GIVE_UP,
	we_vindicator0,
	we_vindicator1,
	we_vindicator2,
	GOOD_FOR_YOU_1,
	come_in_peace,
	GOOD_FOR_YOU_2,
	know_any_jokes,
	JOKE_1,
	better_joke,
	JOKE_2,
	not_very_funny,
	YES_WE_ARE,
	what_about_tpet,
	arilou_told_us,
	bye_zombie,
	GOODBYE_ZOMBIE,
};

#endif /* _STRINGS_H */
