//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	INIT_HELLO,
	who_you,
	WE_ARE_ORZ,
	why_here,
	HERE_BECAUSE,
	ALLIED_HOMEWORLD_HELLO_1,
	ALLIED_HOMEWORLD_HELLO_2,
	ALLIED_HOMEWORLD_HELLO_3,
	ALLIED_HOMEWORLD_HELLO_4,
	ALLIED_SPACE_HELLO_1,
	ALLIED_SPACE_HELLO_2,
	ALLIED_SPACE_HELLO_3,
	ALLIED_SPACE_HELLO_4,
	whats_up_ally,
	GENERAL_INFO_ALLY_1,
	GENERAL_INFO_ALLY_2,
	GENERAL_INFO_ALLY_3,
	GENERAL_INFO_ALLY_4,
	more_about_you,
	ABOUT_US_1,
	ABOUT_US_2,
	ABOUT_US_3,
	ABOUT_US_4,
	where_androsyn,
	DISEMBLE_ABOUT_ANDROSYN,
	must_know_about_androsyn,
	KNOW_TOO_MUCH,
	dont_really_care,
	YOU_ARE_OUR_FRIENDS,
	about_andro_1,
	FORGET_ANDRO_1,
	about_andro_2,
	FORGET_ANDRO_2,
	about_andro_3,
	BLEW_IT,
	NEUTRAL_HOMEWORLD_HELLO_1,
	NEUTRAL_HOMEWORLD_HELLO_2,
	NEUTRAL_HOMEWORLD_HELLO_3,
	NEUTRAL_HOMEWORLD_HELLO_4,
	NEUTRAL_SPACE_HELLO_1,
	NEUTRAL_SPACE_HELLO_2,
	NEUTRAL_SPACE_HELLO_3,
	NEUTRAL_SPACE_HELLO_4,
	hostile_1,
	HOSTILITY_IS_BAD_1,
	hostile_2,
	HOSTILITY_IS_BAD_2,
	we_are_vindicator0,
	we_are_vindicator1,
	we_are_vindicator2,
	NICE_TO_MEET_YOU,
	seem_like_nice_guys,
	ARE_NICE_WANT_ALLY,
	talk_about_alliance,
	OK_TALK_ALLIANCE,
	yes_alliance,
	GREAT,
	no_alliance,
	MAYBE_LATER,
	decide_later,
	OK_LATER,
	why_so_trusting,
	TRUSTING_BECAUSE,
	bye_neutral,
	GOODBYE_NEUTRAL,
	ANGRY_SPACE_HELLO_1,
	ANGRY_SPACE_HELLO_2,
	ANGRY_HOMEWORLD_HELLO_1,
	ANGRY_HOMEWORLD_HELLO_2,
	whats_up_angry,
	GENERAL_INFO_ANGRY_1,
	GENERAL_INFO_ANGRY_2,
	were_sorry,
	APOLOGY_ACCEPTED,
	insult_1,
	insult_2,
	insult_3,
	insult_4,
	insult_5,
	insult_6,
	insult_7,
	insult_8,
	INSULTED_1,
	INSULTED_2,
	INSULTED_3,
	INSULTED_4,
	bye_angry,
	GOODBYE_ANGRY,
	ANGRY_TAALO_HELLO_1,
	ANGRY_TAALO_HELLO_2,
	FRIENDLY_ALLIED_TAALO_HELLO_1,
	FRIENDLY_ALLIED_TAALO_HELLO_2,
	demand_to_land,
	NO_DEMAND,
	ASK_NICELY,
	why_you_here,
	ANGRY_EXPLANATION,
	FRIENDLY_EXPLANATION,
	what_is_this_place,
	FRIENDLY_PLACE,
	ANGRY_PLACE,
	may_we_land,
	SURE_LAND,
	ALLIES_CAN_VISIT,
	make_alliance,
	CANT_ALLY_HERE,
	why_busy,
	BUSY_BECAUSE,
	bye_taalo,
	bye_ally,
	GOODBYE_ALLY,
	FRIENDLY_TAALO_GOODBYE,
	ANGRY_TAALO_GOODBYE,
	HOSTILE_HELLO_1,
	HOSTILE_HELLO_2,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
