//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	killed_fwiffo,
	POOR_FWIFFO,
	fwiffo_fine,
	NOT_LIKELY,
	we_attack_again,
	WE_FIGHT_AGAIN,
	bye_no_ally_offer,
	GOODBYE_NO_ALLY_OFFER,
	bye_angry_spathi,
	GOODBYE_ANGRY_SPATHI,
	why_dont_you_do_it,
	WE_WONT_BECAUSE,
	MEAN_GUYS_RETURN,
	we_apologize,
	DONT_BELIEVE,
	HELLO_AGAIN,
	HATE_YOU_FOREVER,
	WHAT_IS_PASSWORD,
	WHAT_IS_PASSWORD_AGAIN,
	we_are_vindicator0,
	we_are_vindicator1,
	we_are_vindicator2,
	gort_merenga,
	guph_florp,
	pleeese,
	wagngl_fthagn,
	screw_password,
	good_password,
	WRONG_PASSWORD,
	NO_PASSWORD,
	what_do_i_get,
	YOU_GET_TO_LIVE,
	YES_GOOD_PASSWORD,
	spathi_on_pluto,
	WHERE_SPATHI,
	hostage,
	GUN_TO_HEAD,
	we_come_in_peace,
	OF_COURSE,
	KILLED_SPATHI,
	misunderstanding,
	JUST_MISUNDERSTANDING,
//	JUST_MISUNDERSTANDING0,
//	JUST_MISUNDERSTANDING1,
	give_us_resources,
	NO_RESOURCES,
	resources_please,
	SORRY_NO_RESOURCES,
	bye_ally,
	GOODBYE_ALLY,
	what_about_hierarchy,
	what_about_history,
	what_about_alliance,
	what_about_other,
	what_about_precursors,
	enough_info,
	OK_ENOUGH_INFO,
	ABOUT_HIERARCHY,
	ABOUT_HISTORY,
	ABOUT_ALLIANCE,
	ABOUT_OTHER,
	ABOUT_PRECURSORS,
	little_mistake,
	BIG_MISTAKE,
	bye_before_party,
	QUEST_AGAIN,
	GOODBYE_BEFORE_PARTY,
	GOOD_START,
	something_fishy,
	NOTHING_FISHY,
	surrender,
	NO_SURRENDER,
	surrender_or_die,
	DEFEND_OURSELVES,
	hand_in_friendship,
	TOO_AFRAID,
	stronger,
	YOURE_NOT,
	yes_we_are,
	NO_YOURE_NOT,
	how_prove,
	BETTER_IDEA,
	share_info,
	NO_INFO,
	WE_UNDERSTAND,
	prove_strength,
	YOUR_BEHAVIOR,
	what_test,
	BEFORE_ACCEPT,
	WIPE_EVIL,
	think_more,
	COWARD,
	tell_evil,
	i_accept,
	AWAIT_RETURN,
	talk_test,
	already_got_them,
	EARLY_BIRD_CHECK,
	NOT_SURPRISED,
	TEST_AGAIN,
	too_dangerous,
	WE_AGREE,
	HOW_GO_EFFORTS,
	killed_them_all_1,
	killed_them_all_2,
	WILL_CHECK_1,
	WILL_CHECK_2,
	zapped_a_few,
	RETURN_COMPLETE,
	MUST_DESTROY_ALL,
	no_landing,
	saw_creatures,
	YOU_FORTUNATE,
	YOU_LIED_1,
	YOU_LIED_2,
	bye_from_party_1,
	bye_from_party_2,
	bye_from_party_3,
	GOODBYE_FROM_PARTY,
	MUST_PARTY_1,
	MUST_PARTY_2,
	MUST_PARTY_3,
	deals_a_deal,
	WAIT_A_WHILE,
	how_long,
	TEN_YEARS,
	reneging,
	ADULT_VIEW,
	return_beasts,
	WHAT_RELATIONSHIP,
	minds_and_might,
	HUH,
	fellowship,
	WHAT,
	do_as_we_say,
	DEPART_FOR_EARTH,
	HELLO_ALLIES_1,
	HELLO_ALLIES_2,
	HELLO_ALLIES_3,
	whats_up,
	GENERAL_INFO_1,
	GENERAL_INFO_2,
	GENERAL_INFO_3,
	GENERAL_INFO_4,
	GENERAL_INFO_5,
	like_some_info,
	WHAT_ABOUT
};

#endif /* _STRINGS_H */
