/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 * Nota bene: later versions of the GNU General Public License do not apply
 * to this program.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
/*
 * CDP All-interface list (for simplicity)
 */

#ifndef _CDPALLI_H
#define _CDPALLI_H

#include "cdp_iio.h"
#include "cdp_imem.h"
#include "cdp_isnd.h"
#include "cdp_ivid.h"
// TODO: add more cdp_iXXX.h here as they are defined

#endif  /* _CDPALLI_H */
