#define MMRNMHRM_ICON_MASK_PMAP_ANIM 0x00200002L
#define MMRNMHRM_MICON_MASK_PMAP_ANIM 0x00200102L
#define MMRNMHRM_BIG_MASK_PMAP_ANIM 0x00400202L
#define MMRNMHRM_MED_MASK_PMAP_ANIM 0x00400302L
#define MMRNMHRM_SML_MASK_PMAP_ANIM 0x00400402L
#define TORP_BIG_MASK_PMAP_ANIM 0x00400502L
#define TORP_MED_MASK_PMAP_ANIM 0x00400602L
#define TORP_SML_MASK_PMAP_ANIM 0x00400702L
#define YWING_BIG_MASK_PMAP_ANIM 0x00400802L
#define YWING_MED_MASK_PMAP_ANIM 0x00400902L
#define YWING_SML_MASK_PMAP_ANIM 0x00400a02L
#define MMRNMHRM_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


