//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HELLO_BEFORE_AMBUSH_1,
	HELLO_BEFORE_AMBUSH_2,
	HELLO_BEFORE_AMBUSH_3,
	HELLO_BEFORE_AMBUSH_4,
	we_are_vice_squad,
	OK_VICE,
	we_are_the_one_for_you_baby,
	MAYBE_CAPTAIN,
	we_are_vindicator0,
	we_are_vindicator1,
	we_are_vindicator2,
	WELCOME_VINDICATOR0,
	WELCOME_VINDICATOR1,
	WELCOME_VINDICATOR2,
	we_are_impressed,
	SO_AM_I_CAPTAIN,
	HOW_CAN_YOU_BE_HERE,
	we_here_to_help,
	NO_NEED_HELP,
	we_need_help,
	CANT_GIVE_HELP,
	i_need_you,
	OK_NEED,
	i_need_touch_o_vision,
	TOUCH_O_VISION,
	know_about_deep_children,
	WHAT_ABOUT_DEEP_CHILDREN,
	mycons_involved,
	WHAT_PROOF,
	have_no_proof,
	NEED_PROOF,
	have_proof,
	SEE_PROOF,
	look_at_egg_sacks,
	HORRIBLE_TRUTH,
	what_doing_here,
	OUR_NEW_WORLD,
	what_about_war,
	ABOUT_WAR,
	help_us,
	WONT_HELP,
	what_about_history,
	BEFORE_WAR,
	what_about_homeworld,
	ABOUT_HOMEWORLD,
	what_happened,
	DONT_KNOW_HOW,
	what_about_outfit,
	HOPE_YOU_LIKE_IT,
	where_mates,
	MATES_KILLED,
	get_lonely,
	MAKE_OUT_ALL_RIGHT,
	bye,
	GOODBYE,
	MUST_ACT,
	whats_next_step,
	OPEN_VAULT,
	where_is_it,
	DONT_KNOW_WHERE,
	been_there,
	GREAT,
	GIVE_SHUTTLE,
	im_on_my_way,
	doing_this_for_you,
	if_i_die,
	GOOD_LUCK,
	OK_FOUND_VAULT,
	what_now,
	HERES_THE_PLAN,
	whats_my_reward,
	HERES_REWARD,
	bye_after_vault,
	GOODBYE_AFTER_VAULT,
	HELLO_AFTER_AMBUSH_1,
	HELLO_AFTER_AMBUSH_2,
	HELLO_AFTER_AMBUSH_3,
	HELLO_AFTER_AMBUSH_4,
	what_now_after_ambush,
	DO_THIS_AFTER_AMBUSH,
	what_about_you,
	ABOUT_ME,
	whats_up_after_ambush,
	GENERAL_INFO_AFTER_AMBUSH_1,
	GENERAL_INFO_AFTER_AMBUSH_2,
	GENERAL_INFO_AFTER_AMBUSH_3,
	GENERAL_INFO_AFTER_AMBUSH_4,
	bye_after_ambush,
	GOODBYE_AFTER_AMBUSH,
	FOUND_VAULT_YET_1,
	FOUND_VAULT_YET_2,
	vault_hint,
	OK_HINT,
	found_vault,
	bye_before_vault,
	GOODBYE_BEFORE_VAULT,
	what_do_i_get_for_this,
	GRATITUDE,
	not_sure,
	PLEASE,
	READY_FOR_AMBUSH,
	repeat_plan,
	OK_REPEAT_PLAN,
	bye_before_ambush,
	GOODBYE_BEFORE_AMBUSH,
	what_about_us,
	ABOUT_US,
	MORE_COMFORTABLE,
	in_the_spirit,
	OK_SPIRIT,
	what_in_mind,
	SOMETHING_LIKE_THIS,
	hands_off,
	OK_WONT_USE_HANDS,
	why_lights_off,
	LIGHTS_OFF_BECAUSE,
	evil_monster,
	NOT_EVIL_MONSTER,
	disease,
	JUST_RELAX,
	what_happens_if_i_touch_this,
	THIS_HAPPENS,
	are_you_sure_this_is_ok,
	YES_SURE,
	boy_they_never_taught,
	THEN_LET_ME_TEACH,
	not_much_more_to_say,
	THEN_STOP_TALKING,
	LATER,
	SEX_GOODBYE,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
