#define SLYLANDRO_ICON_MASK_PMAP_ANIM 0x00200002L
#define SLYLANDRO_MICON_MASK_PMAP_ANIM 0x00200102L
#define SLYLANDRO_BIG_MASK_PMAP_ANIM 0x00400202L
#define SLYLANDRO_MED_MASK_PMAP_ANIM 0x00400302L
#define SLYLANDRO_SML_MASK_PMAP_ANIM 0x00400402L
#define SLYLANDRO_CAPTAIN_MASK_PMAP_ANIM 0x00400502L


