#define THRADDASH_ICON_MASK_PMAP_ANIM 0x00200002L
#define THRADDASH_MICON_MASK_PMAP_ANIM 0x00200102L
#define THRADDASH_BIG_MASK_PMAP_ANIM 0x00400202L
#define THRADDASH_MED_MASK_PMAP_ANIM 0x00400302L
#define THRADDASH_SML_MASK_PMAP_ANIM 0x00400402L
#define HORN_BIG_MASK_PMAP_ANIM 0x00400502L
#define HORN_MED_MASK_PMAP_ANIM 0x00400602L
#define HORN_SML_MASK_PMAP_ANIM 0x00400702L
#define NAPALM_BIG_MASK_PMAP_ANIM 0x00400802L
#define NAPALM_MED_MASK_PMAP_ANIM 0x00400902L
#define NAPALM_SML_MASK_PMAP_ANIM 0x00400a02L
#define THRADDASH_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


