//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	WE_ARE0,
	WE_ARE1,
	WE_ARE2,
	WE_ARE3,
	WE_ARE4,
	WE_ARE5,
	WE_ARE6,
	WE_ARE7,
	SCOUT_HELLO0,
	SCOUT_HELLO1,
	SCOUT_HELLO2,
	SCOUT_HELLO3,
	INIT_HOME_HELLO0,
	INIT_HOME_HELLO1,
	INIT_HOME_HELLO2,
	INIT_HOME_HELLO3,
	which_fot,
	HE_IS0,
	HE_IS1,
	HE_IS2,
	HE_IS3,
	HE_IS4,
	HE_IS5,
	HE_IS6,
	HE_IS7,
	we_are_vindicator0,
	we_are_vindicator1,
	we_are_vindicator2,
	WE_GLAD0,
	WE_GLAD1,
	WE_GLAD2,
	WE_GLAD3,
	WE_GLAD4,
	WE_GLAD5,
	quiet_toadies,
	TOLD_YOU0,
	TOLD_YOU1,
	TOLD_YOU2,
	TOLD_YOU3,
	TOLD_YOU4,
	TOLD_YOU5,
	TOLD_YOU6,
	TOLD_YOU7,
	your_race,
	YEARS_AGO0,
	YEARS_AGO1,
	YEARS_AGO2,
	YEARS_AGO3,
	YEARS_AGO4,
	YEARS_AGO5,
	YEARS_AGO6,
	YEARS_AGO7,
	YEARS_AGO8,
	YEARS_AGO9,
	YEARS_AGO10,
	YEARS_AGO11,
	YEARS_AGO12,
	YEARS_AGO13,
	where_from,
	TRAVELED_FAR0,
	TRAVELED_FAR1,
	TRAVELED_FAR2,
	TRAVELED_FAR3,
	TRAVELED_FAR4,
	TRAVELED_FAR5,
	what_emergency,
	UNDER_ATTACK0,
	UNDER_ATTACK1,
	UNDER_ATTACK2,
	UNDER_ATTACK3,
	UNDER_ATTACK4,
	UNDER_ATTACK5,
	UNDER_ATTACK6,
	UNDER_ATTACK7,
	UNDER_ATTACK8,
	UNDER_ATTACK9,
	UNDER_ATTACK10,
	UNDER_ATTACK11,
	tough_luck,
	NOT_HELPFUL0,
	NOT_HELPFUL1,
	NOT_HELPFUL2,
	NOT_HELPFUL3,
	NOT_HELPFUL4,
	NOT_HELPFUL5,
	what_look_like,
	LOOK_LIKE0,
	LOOK_LIKE1,
	LOOK_LIKE2,
	LOOK_LIKE3,
	valuable_info,
	GOODBYE0,
	GOODBYE1,
	GOODBYE2,
	GOODBYE3,
	all_very_interesting,
	SEE_TOLD_YOU0,
	SEE_TOLD_YOU1,
	SEE_TOLD_YOU2,
	SEE_TOLD_YOU3,
	how_can_i_help,
	ALLY_WITH_US0,
	ALLY_WITH_US1,
	ALLY_WITH_US2,
	ALLY_WITH_US3,
	ALLY_WITH_US4,
	ALLY_WITH_US5,
	decide_later,
	PLEASE_HURRY0,
	PLEASE_HURRY1,
	EMMISSARIES0,
	EMMISSARIES1,
	EMMISSARIES2,
	EMMISSARIES3,
	EMMISSARIES4,
	EMMISSARIES5,
	EMMISSARIES6,
	EMMISSARIES7,
	sure,
	WE_ALLY0,
	WE_ALLY1,
	WE_ALLY2,
	WE_ALLY3,
	WE_ALLY4,
	WE_ALLY5,
	never,
	WE_ENEMIES0,
	WE_ENEMIES1,
	HOSTILE_HELLO_10,
	HOSTILE_HELLO_11,
	HOSTILE_HELLO_20,
	HOSTILE_HELLO_21,
	HOSTILE_HELLO_22,
	HOSTILE_HELLO_23,
	HOSTILE_HELLO_24,
	HOSTILE_HELLO_25,
	HOSTILE_HELLO_30,
	HOSTILE_HELLO_31,
	HOSTILE_HELLO_40,
	HOSTILE_HELLO_41,
	NEUTRAL_HOME_HELLO_10,
	NEUTRAL_HOME_HELLO_11,
	NEUTRAL_HOME_HELLO_12,
	NEUTRAL_HOME_HELLO_13,
	NEUTRAL_HOME_HELLO_20,
	NEUTRAL_HOME_HELLO_21,
	NEUTRAL_HOME_HELLO_22,
	NEUTRAL_HOME_HELLO_23,
	ALLIED_HOME_HELLO_10,
	ALLIED_HOME_HELLO_11,
	ALLIED_HOME_HELLO_12,
	ALLIED_HOME_HELLO_13,
	ALLIED_HOME_HELLO_20,
	ALLIED_HOME_HELLO_21,
	ALLIED_HOME_HELLO_22,
	ALLIED_HOME_HELLO_23,
	ALLIED_HOME_HELLO_24,
	ALLIED_HOME_HELLO_25,
	ALLIED_HOME_HELLO_26,
	ALLIED_HOME_HELLO_27,
	ALLIED_HOME_HELLO_30,
	ALLIED_HOME_HELLO_31,
	ALLIED_HOME_HELLO_40,
	ALLIED_HOME_HELLO_41,
	THANKS_FOR_RESCUE0,
	THANKS_FOR_RESCUE1,
	THANKS_FOR_RESCUE2,
	THANKS_FOR_RESCUE3,
	THANKS_FOR_RESCUE4,
	THANKS_FOR_RESCUE5,
	THANKS_FOR_RESCUE6,
	THANKS_FOR_RESCUE7,
	THANKS_FOR_RESCUE8,
	THANKS_FOR_RESCUE9,
	THANKS_FOR_RESCUE10,
	THANKS_FOR_RESCUE11,
	bye_homeworld,
	GOODBYE_HOME0,
	GOODBYE_HOME1,
	whats_up_homeworld,
	GENERAL_INFO_10,
	GENERAL_INFO_11,
	GENERAL_INFO_12,
	GENERAL_INFO_13,
	GENERAL_INFO_20,
	GENERAL_INFO_21,
	GENERAL_INFO_22,
	GENERAL_INFO_23,
	GENERAL_INFO_24,
	GENERAL_INFO_25,
	GENERAL_INFO_26,
	GENERAL_INFO_27,
	GENERAL_INFO_30,
	GENERAL_INFO_31,
	GENERAL_INFO_32,
	GENERAL_INFO_33,
	GENERAL_INFO_34,
	GENERAL_INFO_35,
	GENERAL_INFO_40,
	GENERAL_INFO_41,
	GENERAL_INFO_42,
	GENERAL_INFO_43,
	GENERAL_INFO_44,
	GENERAL_INFO_45,
	GENERAL_INFO_46,
	GENERAL_INFO_47,
	GENERAL_INFO_48,
	GENERAL_INFO_49,
	GENERAL_INFO_410,
	GENERAL_INFO_411,
	any_war_news,
	UTWIG_DELAY0,
	UTWIG_DELAY1,
	UTWIG_DELAY2,
	UTWIG_DELAY3,
	UTWIG_DELAY4,
	UTWIG_DELAY5,
	UTWIG_DELAY6,
	UTWIG_DELAY7,
	UTWIG_DELAY8,
	UTWIG_DELAY9,
	UTWIG_DELAY10,
	UTWIG_DELAY11,
	UTWIG_DELAY12,
	UTWIG_DELAY13,
	KOHRAH_WINNING0,
	KOHRAH_WINNING1,
	KOHRAH_WINNING2,
	KOHRAH_WINNING3,
	KOHRAH_WINNING4,
	KOHRAH_WINNING5,
	KOHRAH_WINNING6,
	KOHRAH_WINNING7,
	KOHRAH_WINNING8,
	KOHRAH_WINNING9,
	URQUAN_NEARLY_GONE0,
	URQUAN_NEARLY_GONE1,
	URQUAN_NEARLY_GONE2,
	URQUAN_NEARLY_GONE3,
	URQUAN_NEARLY_GONE4,
	URQUAN_NEARLY_GONE5,
	KOHRAH_FRENZY0,
	KOHRAH_FRENZY1,
	KOHRAH_FRENZY2,
	KOHRAH_FRENZY3,
	KOHRAH_FRENZY4,
	KOHRAH_FRENZY5,
	KOHRAH_FRENZY6,
	KOHRAH_FRENZY7,
	KOHRAH_FRENZY8,
	KOHRAH_FRENZY9,
	KOHRAH_FRENZY10,
	KOHRAH_FRENZY11,
	NO_WAR_NEWS0,
	NO_WAR_NEWS1,
	i_want_alliance,
	GOOD0,
	GOOD1,
	GOOD2,
	GOOD3,
	GOOD4,
	GOOD5,
	GOOD6,
	GOOD7,
	GOOD8,
	GOOD9,
	want_specific_info,
	WHAT_SPECIFIC_INFO0,
	WHAT_SPECIFIC_INFO1,
	enough_info,
	OK_ENOUGH_INFO,
	what_about_others,
	ABOUT_OTHERS0,
	ABOUT_OTHERS1,
	ABOUT_OTHERS2,
	ABOUT_OTHERS3,
	ABOUT_OTHERS4,
	ABOUT_OTHERS5,
	ABOUT_OTHERS6,
	ABOUT_OTHERS7,
	ABOUT_OTHERS8,
	ABOUT_OTHERS9,
	ABOUT_OTHERS10,
	ABOUT_OTHERS11,
	ABOUT_OTHERS12,
	ABOUT_OTHERS13,
	what_about_zebranky,
	ABOUT_ZEBRANKY0,
	ABOUT_ZEBRANKY1,
	ABOUT_ZEBRANKY2,
	ABOUT_ZEBRANKY3,
	ABOUT_ZEBRANKY4,
	ABOUT_ZEBRANKY5,
	ABOUT_ZEBRANKY6,
	ABOUT_ZEBRANKY7,
	what_about_past,
	ABOUT_PAST0,
	ABOUT_PAST1,
	ABOUT_PAST2,
	ABOUT_PAST3,
	ABOUT_PAST4,
	ABOUT_PAST5,
	ABOUT_PAST6,
	ABOUT_PAST7,
	ABOUT_PAST8,
	ABOUT_PAST9,
	ABOUT_PAST10,
	ABOUT_PAST11,
	what_about_stinger,
	ABOUT_STINGER0,
	ABOUT_STINGER1,
	ABOUT_STINGER2,
	ABOUT_STINGER3,
	ABOUT_STINGER4,
	ABOUT_STINGER5,
	what_about_guy_in_back,
	ABOUT_GUY0,
	ABOUT_GUY1,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	OUT_TAKES0,
	OUT_TAKES1,
	OUT_TAKES2,
	OUT_TAKES3,
	OUT_TAKES4,
	OUT_TAKES5,
	OUT_TAKES6,
	OUT_TAKES7,
	OUT_TAKES8,
	OUT_TAKES9,
	OUT_TAKES10,
	OUT_TAKES11,
	OUT_TAKES12,
	OUT_TAKES13,
};

#endif /* _STRINGS_H */

