#define STARCON_COLOR_MAP 0x00200004L
#define STARCON_GAME_STRINGS 0x03800104L
#define HYPER_COLOR_TAB 0x04800204L
#define ARISPACE_COLOR_TAB 0x04a00304L
#define ORBPLAN_COLOR_MAP 0x05200404L
#define OOLITE_COLOR_TAB 0x09000504L
#define OOLITE_XLAT_TAB 0x09000604L
#define YTTRIC_COLOR_TAB 0x09200704L
#define YTTRIC_XLAT_TAB 0x09400804L
#define QUASI_DEGENERATE_COLOR_TAB 0x09600904L
#define QUASI_DEGENERATE_XLAT_TAB 0x09800a04L
#define LANTHANIDE_COLOR_TAB 0x09a00b04L
#define TREASURE_COLOR_TAB 0x09c00c04L
#define UREA_COLOR_TAB 0x09e00d04L
#define UREA_XLAT_TAB 0x0a000e04L
#define METAL_COLOR_TAB 0x0a200f04L
#define METAL_XLAT_TAB 0x0a201004L
#define RADIOACTIVE_COLOR_TAB 0x0a401104L
#define OPALESCENT_COLOR_TAB 0x0a601204L
#define OPALESCENT_XLAT_TAB 0x0a801304L
#define CYANIC_COLOR_TAB 0x0aa01404L
#define ACID_COLOR_TAB 0x0ac01504L
#define ALKALI_COLOR_TAB 0x0ae01604L
#define HALIDE_COLOR_TAB 0x0b001704L
#define GREEN_COLOR_TAB 0x0b201804L
#define COPPER_COLOR_TAB 0x0b401904L
#define CARBIDE_COLOR_TAB 0x0b601a04L
#define ULTRAMARINE_COLOR_TAB 0x0b801b04L
#define NOBLE_COLOR_TAB 0x0ba01c04L
#define AZURE_COLOR_TAB 0x0bc01d04L
#define CHONDRITE_COLOR_TAB 0x0be01e04L
#define CHONDRITE_XLAT_TAB 0x0be01f04L
#define PURPLE_COLOR_TAB 0x0c002004L
#define SUPER_DENSE_COLOR_TAB 0x0c202104L
#define PELLUCID_COLOR_TAB 0x0c402204L
#define DUST_COLOR_TAB 0x0c602304L
#define MAROON_COLOR_TAB 0x0c802404L
#define CIMMERIAN_COLOR_TAB 0x0ca02504L
#define INFRARED_COLOR_TAB 0x0cc02604L
#define SELENIC_COLOR_TAB 0x0ce02704L
#define AURIC_COLOR_TAB 0x0d002804L
#define FLUORESCENT_COLOR_TAB 0x0d202904L
#define ULTRAVIOLET_COLOR_TAB 0x0d402a04L
#define PLUTONIC_COLOR_TAB 0x0d602b04L
#define RAINBOW_COLOR_TAB 0x0d802c04L
#define RAINBOW_XLAT_TAB 0x0d802d04L
#define CRACKED_COLOR_TAB 0x0da02e04L
#define CRACKED_XLAT_TAB 0x0da02f04L
#define SAPPHIRE_COLOR_TAB 0x0dc03004L
#define SAPPHIRE_XLAT_TAB 0x0de03104L
#define ORGANIC_COLOR_TAB 0x0e003204L
#define XENOLITHIC_COLOR_TAB 0x0e203304L
#define REDUX_COLOR_TAB 0x0e403404L
#define REDUX_XLAT_TAB 0x0e403504L
#define PRIMORDIAL_COLOR_TAB 0x0e603604L
#define EMERALD_COLOR_TAB 0x0e803704L
#define CHLORINE_COLOR_TAB 0x0ea03804L
#define CHLORINE_XLAT_TAB 0x0ec03904L
#define MAGNETIC_COLOR_TAB 0x0ee03a04L
#define WATER_COLOR_TAB 0x0f003b04L
#define TELLURIC_COLOR_TAB 0x0f203c04L
#define HYDROCARBON_COLOR_TAB 0x0f403d04L
#define IODINE_COLOR_TAB 0x0f603e04L
#define VINYLOGOUS_COLOR_TAB 0x0f803f04L
#define RUBY_COLOR_TAB 0x0fa04004L
#define MAGMA_COLOR_TAB 0x0fc04104L
#define CRIMSON_COLOR_TAB 0x0fe04204L
#define GAS_XLAT_TAB 0x10004304L
#define BLU_GAS_COLOR_TAB 0x10204404L
#define CYA_GAS_COLOR_TAB 0x10404504L
#define GRN_GAS_COLOR_TAB 0x10604604L
#define GRY_GAS_COLOR_TAB 0x10804704L
#define ORA_GAS_COLOR_TAB 0x10a04804L
#define PUR_GAS_COLOR_TAB 0x10c04904L
#define RED_GAS_COLOR_TAB 0x10e04a04L
#define VIO_GAS_COLOR_TAB 0x11004b04L
#define YEL_GAS_COLOR_TAB 0x11204c04L
#define MOONBASE_STRTAB 0x11604d04L
#define MAIDENS_STRTAB 0x11804e04L
#define CHMMR_BASE_STRTAB 0x11a04f04L
#define AQUA_STRTAB 0x11c05004L
#define BURV_BCS_STRTAB 0x11e05104L
#define TAALO_DEVICE_STRTAB 0x12005204L
#define SUN_DEVICE_STRTAB 0x12205304L
#define VAULT_STRTAB 0x12405404L
#define WRECK_STRTAB 0x12605504L
#define BOMB_STRTAB 0x12805604L
#define BEAST_STRTAB 0x12a05704L
#define EGG_CASE_STRTAB 0x12c05804L
#define SPAPLUTO_STRTAB 0x12e05904L
#define BURV_RUINS_STRTAB 0x13005a04L
#define ANDROSYNTH_RUINS_STRTAB 0x13205b04L
#define DRUUGE_RUINS_STRTAB 0x13405c04L
#define PKUNK_RUINS_STRTAB 0x13605d04L
#define RUINS_STRTAB 0x13a05e04L
#define UMGAH_BCS_STRTAB 0x13c05f04L

#define IPSUN_COLOR_MAP 0x05206004L
#define HANGAR_COLOR_TAB 0x18a06104L
#define INTROPRES_STRTAB 0x23c06204L
#define FINALPRES_STRTAB 0x23e06304L
#define CREDITS_STRTAB 0x24206404L
#define JOYSTICK_ALPHA_STRTAB 0x24406504L
#define SETUP_MENU_STRTAB 0x23a11f04L
