#define MYCON_ICON_MASK_PMAP_ANIM 0x00200002L
#define MYCON_MICON_MASK_PMAP_ANIM 0x00200102L
#define MYCON_BIG_MASK_PMAP_ANIM 0x00400202L
#define MYCON_MED_MASK_PMAP_ANIM 0x00400302L
#define MYCON_SML_MASK_PMAP_ANIM 0x00400402L
#define PLASMA_BIG_MASK_PMAP_ANIM 0x00400502L
#define PLASMA_MED_MASK_PMAP_ANIM 0x00400602L
#define PLASMA_SML_MASK_PMAP_ANIM 0x00400702L
#define MYCON_CAPTAIN_MASK_PMAP_ANIM 0x00400802L


