//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	ZEX_HELLO_1,
	ZEX_HELLO_2,
	ZEX_HELLO_3,
	ZEX_HELLO_4,
	FIGHT_OR_TRADE_1,
	FIGHT_OR_TRADE_2,
	what_you_do_here,
	MY_MENAGERIE,
	what_about_menagerie,
	NEED_NEW_CREATURE,
	what_about_creature,
	ABOUT_CREATURE,
	about_creature_again,
	CREATURE_AGAIN,
	i_have_beast,
	GIVE_BEAST,
	ok_take_beast,
	FOOL_AIEE0,
	FOOL_AIEE1,
	why_trust_1,
	TRUST_1,
	why_trust_2,
	TRUST_2,
	why_trust_3,
	TRUST_3,
	why_dont_you_attack,
	LIKE_YOU,
	why_like_me,
	LIKE_BECAUSE,
	are_you_a_pervert,
	CALL_ME_WHAT_YOU_WISH,
	take_by_force,
	PRECURSOR_DEVICE,
	regardless,
	THEN_FIGHT,
	you_lied,
	YUP_LIED,
	kill_you,
	FIGHT_AGAIN,
	bye_zex,
	GOODBYE_ZEX,
	HOMEWORLD_HELLO_1,
	HOMEWORLD_HELLO_2,
	HOMEWORLD_HELLO_3,
	HOMEWORLD_HELLO_4,
	SPACE_HELLO_1,
	SPACE_HELLO_2,
	SPACE_HELLO_3,
	SPACE_HELLO_4,
	kill_you_squids_1,
	kill_you_squids_2,
	kill_you_squids_3,
	kill_you_squids_4,
	WE_FIGHT,
	why_so_mean,
	URQUAN_SLAVES,
	deeper_reason,
	OLD_INSULT,
	if_we_apologize,
	PROBABLY_NOT,
	try_any_way,
	NOPE,
	APOLOGIZE_IN_SPACE,
	apology_1,
	NOT_ACCEPTED_1,
	apology_2,
	NOT_ACCEPTED_2,
	apology_3,
	NOT_ACCEPTED_3,
	apology_4,
	NOT_ACCEPTED_4,
	apology_5,
	NOT_ACCEPTED_5,
	apology_6,
	NOT_ACCEPTED_6,
	apology_7,
	NOT_ACCEPTED_7,
	apology_8,
	NOT_ACCEPTED_8,
	apology_9,
	NOT_ACCEPTED_9,
	apology_10,
	TRUTH,
	whats_up_hostile,
	GENERAL_INFO_HOSTILE_1,
	GENERAL_INFO_HOSTILE_2,
	GENERAL_INFO_HOSTILE_3,
	GENERAL_INFO_HOSTILE_4,
	cant_we_be_friends_1,
	NEVER_UGLY_HUMANS_1,
	cant_we_be_friends_2,
	NEVER_UGLY_HUMANS_2,
	cant_we_be_friends_3,
	NEVER_UGLY_HUMANS_3,
	cant_we_be_friends_4,
	NEVER_UGLY_HUMANS_4,
	bye_hostile_space,
	GOODBYE_AND_DIE_HOSTILE_SPACE_1,
	GOODBYE_AND_DIE_HOSTILE_SPACE_2,
	GOODBYE_AND_DIE_HOSTILE_SPACE_3,
	GOODBYE_AND_DIE_HOSTILE_SPACE_4,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
