#define SIS_ICON_MASK_PMAP_ANIM 0x00400002L
#define SIS_BIG_MASK_PMAP_ANIM 0x00400102L
#define SIS_MED_MASK_PMAP_ANIM 0x00400202L
#define SIS_SML_MASK_PMAP_ANIM 0x00400302L
#define BLASTER_BIG_MASK_PMAP_ANIM 0x00400402L
#define BLASTER_MED_MASK_PMAP_ANIM 0x00400502L
#define BLASTER_SML_MASK_PMAP_ANIM 0x00400602L
#define SIS_CAPTAIN_MASK_PMAP_ANIM 0x00400702L
#define SIS_HYPER_MASK_PMAP_ANIM 0x00600802L


