//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	AMBUSH_IS_FIRST_HELLO,
	INIT_BOMB_WORLD_HELLO,
	SUBSEQ_BOMB_WORLD_HELLO,
	whats_up_at_bomb_planet,
	GEN_INFO_AT_BOMB_PLANET,
	we_get_bomb,
	NOT_GET_BOMB,
	then_we_take_bomb,
	FIGHT_FOR_BOMB,
	GOODBYE_FROM_BOMB_PLANET,
	NOT_ENOUGH_ROOM,
	TRADE_FOR_SPHERE,
	no_way,
	OK_REGULAR_DEAL,
	way,
	OK_HERES_SPHERE,
	whats_the_sphere_again,
	SPHERE_IS,
	WE_SELL_FOR_CREW,
	i_will_never_trade_crew,
	YOUR_LOSS,
	isnt_this_slave_trading,
	NO_SLAVE_TRADE,
	what_do_with_crew,
	HAVE_FUN,
	im_ready_to_buy,
	THIS_FOR_SALE,
	HAVE_SPHERE,
	HAVE_ART_2,
	HAVE_ART_1,
	SHIPS_AND_FUEL,
	BOUGHT_SHIP,
	BOUGHT_FUEL,
	BOUGHT_ART_2,
	BOUGHT_ART_1,
	BOUGHT_SPHERE,
	repeat_what_to_sell,
	INIT_SPACE_HELLO,
	SUBSEQUENT_SPACE_HELLO,
	whats_up_in_space,
	GENERAL_INFO_IN_SPACE_1,
	GENERAL_INFO_IN_SPACE_2,
	GENERAL_INFO_IN_SPACE_3,
	GENERAL_INFO_IN_SPACE_4,
	GOODBYE_FROM_SPACE,
	HSTL_TRADE_WORLD_HELLO_1,
	HSTL_TRADE_WORLD_HELLO_2,
	HOSTILE_SPACE_HELLO_1,
	HOSTILE_SPACE_HELLO_2,
	INITIAL_TRADE_WORLD_HELLO,
	SSQ_TRADE_WORLD_HELLO_1,
	SSQ_TRADE_WORLD_HELLO_2,
	SSQ_TRADE_WORLD_HELLO_3,
	SSQ_TRADE_WORLD_HELLO_4,
	whats_up_at_trade_world,
	GEN_INFO_AT_TRADE_WORLD_1,
	GEN_INFO_AT_TRADE_WORLD_2,
	GEN_INFO_AT_TRADE_WORLD_3,
	GEN_INFO_AT_TRADE_WORLD_4,
	SCAN_MAIDENS,
	SCAN_FRAGMENTS,
	SCAN_DRUUGE_CASTER,
	SCAN_ARILOU_SPAWNER,
	ENOUGH_FRAGMENTS,
	READY_TO_BUY,
	READY_TO_SELL,
	BYE_FROM_TRADE_WORLD_1,
	BYE_FROM_TRADE_WORLD_2,
	NOT_ENOUGH_CREW,
	EXCHANGE_MADE,
	OK_DONE_BUYING,
	OK_DONE_SELLING,
	bye,
	want_to_sell,
	want_to_buy,
	buy_druuge_ship,
	buy_fuel,
	buy_art_1,
	buy_art_2,
	buy_rosy_sphere,
	done_buying,
	done_selling,
	sell_maidens,
	sell_caster,
	sell_fragments,
	sell_spawner,
	BOUGHT_MAIDENS,
	BOUGHT_FRAGMENTS,
	BOUGHT_CASTER,
	YOU_GET,
	YOU_ALSO_GET,
	BOUGHT_SPAWNER,
	SALVAGE_YOUR_SHIP_1,
	SALVAGE_YOUR_SHIP_2,
	DEAL_FOR_STATED_SHIPS,
	DEAL_FOR_LESS_SHIPS,
	DEAL_FOR_NO_SHIPS,
	FUEL0,
	FUEL1,
	HIDEOUS_DEAL,
	BAD_DEAL,
	FAIR_DEAL,
	GOOD_DEAL,
	FINE_DEAL,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
