#ifndef _ISTRTAB_H
#define _ISTRTAB_H

#define ARILOU_RACE_STRINGS 0x00200004L
#define ARILOU_SHIP_SOUNDS 0x00400104L
#define ARILOU_VICTORY_SONG 0x00600204L

typedef RESOURCE STRTAB_INSTANCE;

#endif /* _ISTRTAB_H */
