#define ARILOU_ICON_MASK_PMAP_ANIM 0x00200002L
#define ARILOU_MICON_MASK_PMAP_ANIM 0x00200102L
#define ARILOU_BIG_MASK_PMAP_ANIM 0x00400202L
#define ARILOU_MED_MASK_PMAP_ANIM 0x00400302L
#define ARILOU_SML_MASK_PMAP_ANIM 0x00400402L
#define WARP_BIG_MASK_PMAP_ANIM 0x00400502L
#define WARP_MED_MASK_PMAP_ANIM 0x00400602L
#define WARP_SML_MASK_PMAP_ANIM 0x00400702L
#define ARILOU_CAPTAIN_MASK_PMAP_ANIM 0x00400802L


