//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

enum
{
	INIT_PACKAGE = 1,
	TITLE_PACKAGE,
	ARILOU_SHIP_PACKAGE,
	CHMMR_SHIP_PACKAGE,
	EARTHLING_SHIP_PACKAGE,
	ORZ_SHIP_PACKAGE,
	PKUNK_SHIP_PACKAGE,
	SHOFIXTI_SHIP_PACKAGE,
	SPATHI_SHIP_PACKAGE,
	SUPOX_SHIP_PACKAGE,
	THRADD_SHIP_PACKAGE,
	UTWIG_SHIP_PACKAGE,
	VUX_SHIP_PACKAGE,
	YEHAT_SHIP_PACKAGE,
	MELNORME_SHIP_PACKAGE,
	DRUUGE_SHIP_PACKAGE,
	ILWRATH_SHIP_PACKAGE,
	MYCON_SHIP_PACKAGE,
	SLYLANDRO_SHIP_PACKAGE,
	UMGAH_SHIP_PACKAGE,
	URQUAN_SHIP_PACKAGE,
	ZOQFOTPIK_SHIP_PACKAGE,
	SYREEN_SHIP_PACKAGE,
	BLACKURQ_SHIP_PACKAGE,
	ANDROSYN_SHIP_PACKAGE,
	CHENJESU_SHIP_PACKAGE,
	MMRNMHRM_SHIP_PACKAGE,
	KERNEL_PACKAGE,
	SC2KERNEL_PACKAGE,
	MICROFONT_PACKAGE,
	SC2DATA_PACKAGE,
	MENU_SOUND_PACKAGE,
	SOUND_PACKAGE,
	SPACE_PACKAGE,
	AMBIENT_HYPER_PACKAGE,
	HYPER_PACKAGE,
	ARISPACE_PACKAGE,
	HYPERSPACE_MUSIC_PACKAGE,
	QUASISPACE_MUSIC_PACKAGE,
	INIT_PACKAGE,
	IP_PACKAGE,
	LANDER_PACKAGE,
	ORBIT_VIEW_PACKAGE,
	EARTH_PACKAGE,
	CANNISTER_PACKAGE,
	LIFE00_PACKAGE,
	LIFE01_PACKAGE,
	LIFE02_PACKAGE,
	LIFE03_PACKAGE,
	LIFE04_PACKAGE,
	LIFE05_PACKAGE,
	LIFE06_PACKAGE,
	LIFE07_PACKAGE,
	LIFE08_PACKAGE,
	LIFE09_PACKAGE,
	LIFE10_PACKAGE,
	LIFE11_PACKAGE,
	LIFE12_PACKAGE,
	LIFE13_PACKAGE,
	LIFE14_PACKAGE,
	LIFE15_PACKAGE,
	LIFE16_PACKAGE,
	LIFE17_PACKAGE,
	LIFE18_PACKAGE,
	LIFE19_PACKAGE,
	LIFE20_PACKAGE,
	LIFE21_PACKAGE,
	LIFE22_PACKAGE,
	LIFE23_PACKAGE,
	LIFE24_PACKAGE,
	LIFE25_PACKAGE,
	OOLITE_PACKAGE,
	YTTRIC_COLOR_PACKAGE,
	YTTRIC_XLAT_PACKAGE,
	QD_COLOR_PACKAGE,
	QD_XLAT_PACKAGE,
	LANTHANIDE_COLOR_PACKAGE,
	TREASURE_COLOR_PACKAGE,
	UREA_COLOR_PACKAGE,
	UREA_XLAT_PACKAGE,
	METAL_PACKAGE,
	RADIOACTIVE_COLOR_PACKAGE,
	OPALESCENT_COLOR_PACKAGE,
	OPALESCENT_XLAT_PACKAGE,
	CYANIC_COLOR_PACKAGE,
	ACID_COLOR_PACKAGE,
	ALKALI_COLOR_PACKAGE,
	HALIDE_COLOR_PACKAGE,
	GREEN_COLOR_PACKAGE,
	COPPER_COLOR_PACKAGE,
	CARBIDE_COLOR_PACKAGE,
	ULTRAMARINE_COLOR_PACKAGE,
	NOBLE_COLOR_PACKAGE,
	AZURE_COLOR_PACKAGE,
	CHONDRITE_PACKAGE,
	PURPLE_COLOR_PACKAGE,
	SUPER_DENSE_COLOR_PACKAGE,
	PELLUCID_COLOR_PACKAGE,
	DUST_COLOR_PACKAGE,
	MAROON_COLOR_PACKAGE,
	CIMMERIAN_COLOR_PACKAGE,
	INFRARED_COLOR_PACKAGE,
	SELENIC_COLOR_PACKAGE,
	AURIC_COLOR_PACKAGE,
	FLUORESCENT_COLOR_PACKAGE,
	ULTRAVIOLET_COLOR_PACKAGE,
	PLUTONIC_COLOR_PACKAGE,
	RAINBOW_COLOR_PACKAGE,
	CRACKED_PACKAGE,
	SAPPHIRE_COLOR_PACKAGE,
	SAPPHIRE_XLAT_PACKAGE,
	ORGANIC_COLOR_PACKAGE,
	XENOLITHIC_COLOR_PACKAGE,
	REDUX_PACKAGE,
	PRIMORDIAL_COLOR_PACKAGE,
	EMERALD_COLOR_PACKAGE,
	CHLORINE_COLOR_PACKAGE,
	CHLORINE_XLAT_PACKAGE,
	MAGNETIC_COLOR_PACKAGE,
	WATER_COLOR_PACKAGE,
	TELLURIC_COLOR_PACKAGE,
	HYDROCARBON_COLOR_PACKAGE,
	IODINE_COLOR_PACKAGE,
	VINYLOGOUS_COLOR_PACKAGE,
	RUBY_COLOR_PACKAGE,
	MAGMA_COLOR_PACKAGE,
	CRIMSON_COLOR_PACKAGE,
	GAS_XLAT_PACKAGE,
	BLU_GAS_CT_PACKAGE,
	CYA_GAS_CT_PACKAGE,
	GRN_GAS_CT_PACKAGE,
	GRY_GAS_CT_PACKAGE,
	ORA_GAS_CT_PACKAGE,
	PUR_GAS_CT_PACKAGE,
	RED_GAS_CT_PACKAGE,
	VIO_GAS_CT_PACKAGE,
	YEL_GAS_CT_PACKAGE,
	REPORT_PACKAGE,
	MOONBASE_PACKAGE,
	MAIDENS_PACKAGE,
	CHMMR_BASE_PACKAGE,
	AQUA_PACKAGE,
	BURV_BCS_PACKAGE,
	TAALO_DEVICE_PACKAGE,
	SUN_DEVICE_PACKAGE,
	VAULT_PACKAGE,
	WRECK_PACKAGE,
	BOMB_PACKAGE,
	BEAST_PACKAGE,
	EGG_CASE_PACKAGE,
	SPAPLUTO_PACKAGE,
	BURV_RUINS_PACKAGE,
	ANDROSYNTH_RUINS_PACKAGE,
	DRUUGE_RUINS_PACKAGE,
	PKUNK_RUINS_PACKAGE,
	RACE_RUINS_PACKAGE,
	RACE_RUINS_TEXT_PACKAGE,
	UMGAH_BCS_PACKAGE,
	IP_MUSIC_PACKAGE,
	ORBIT_MUSIC_PACKAGE,
	MELEE_BGROUND_PACKAGE,
	MELEE_PICK_PACKAGE,
	SC2_SEGUE_PACKAGE,
	SC2_PICK_PACKAGE,
	ARILOU_CONVERSATION_PACKAGE,
	CHMMR_CONVERSATION_PACKAGE,
	COMMANDER_CONVERSATION_PACKAGE,
	ORZ_CONVERSATION_PACKAGE,
	PKUNK_CONVERSATION_PACKAGE,
	SHOFIXTI_CONVERSATION_PACKAGE,
	SPATHI_CONVERSATION_PACKAGE,
	SUPOX_CONVERSATION_PACKAGE,
	THRADD_CONVERSATION_PACKAGE,
	UTWIG_CONVERSATION_PACKAGE,
	VUX_CONVERSATION_PACKAGE,
	YEHAT_CONVERSATION_PACKAGE,
	MELNORME_CONVERSATION_PACKAGE,
	DRUUGE_CONVERSATION_PACKAGE,
	ILWRATH_CONVERSATION_PACKAGE,
	MYCON_CONVERSATION_PACKAGE,
	SLYLANDRO_CONVERSATION_PACKAGE,
	UMGAH_CONVERSATION_PACKAGE,
	URQUAN_CONVERSATION_PACKAGE,
	ZOQFOTPIK_CONVERSATION_PACKAGE,
	SYREEN_CONVERSATION_PACKAGE,
	BLACKURQ_CONVERSATION_PACKAGE,
	TALKING_PET_CONVERSATION_PACKAGE,
	SLYLANDRO_HOME_CONVERSATION_PACKAGE,
	PLAYER_CONVERSATION_PACKAGE,
	SIS_PACKAGE,
	SAMATRA_PACKAGE,
	URQUAN_PROBE_PACKAGE,
	RESTART_PACKAGE,
	MODULE_PACKAGE,
	OUTFIT_PACKAGE,
	OUTFIT_MUSIC_PACKAGE,
	BLUEPRINT_PACKAGE,
	SHIPYARD_MUSIC_PACKAGE,
	STARBASE_PACKAGE,
	SAMATRA_BATTLE_PACKAGE,
	SHIELDED_BATTLE_PACKAGE,
	PLANET00_PACKAGE,
	PLANET01_PACKAGE,
	PLANET02_PACKAGE,
	PLANET03_PACKAGE,
	PLANET04_PACKAGE,
	PLANET05_PACKAGE,
	PLANET06_PACKAGE,
	PLANET07_PACKAGE,
	PLANET08_PACKAGE,
	PLANET09_PACKAGE,
	PLANET10_PACKAGE,
	PLANET11_PACKAGE,
	PLANET12_PACKAGE,
	PLANET13_PACKAGE,
	PLANET14_PACKAGE,
	PLANET15_PACKAGE,
	PLANET16_PACKAGE,
	PLANET17_PACKAGE,
	PLANET18_PACKAGE,
	PLANET19_PACKAGE,
	PLANET20_PACKAGE,
	PLANET21_PACKAGE,
	PLANET22_PACKAGE,
	PLANET23_PACKAGE,
	PLANET24_PACKAGE,
	PLANET25_PACKAGE,
	PLANET26_PACKAGE,
	PLANET27_PACKAGE,
	PLANET28_PACKAGE,
	PLANET29_PACKAGE,
	PLANET30_PACKAGE,
	PLANET31_PACKAGE,
	PLANET32_PACKAGE,
	PLANET33_PACKAGE,
	PLANET34_PACKAGE,
	PLANET35_PACKAGE,
	PLANET36_PACKAGE,
	PLANET37_PACKAGE,
	PLANET38_PACKAGE,
	PLANET39_PACKAGE,
	PLANET40_PACKAGE,
	PLANET41_PACKAGE,
	PLANET42_PACKAGE,
	PLANET43_PACKAGE,
	PLANET44_PACKAGE,
	PLANET45_PACKAGE,
	PLANET46_PACKAGE,
	PLANET47_PACKAGE,
	PLANET48_PACKAGE,
	PLANET49_PACKAGE,
	PLANET50_PACKAGE,
	PLANET51_PACKAGE,
	PLANET52_PACKAGE,
	PLANET53_PACKAGE,
	PLANET54_PACKAGE,
	PLANET55_PACKAGE,
	PLANET56_PACKAGE,
	PLANET57_PACKAGE,
	PLANET58_PACKAGE,
	BATTLE_MUSIC_PACKAGE,
	CREDITBKGD_PACKAGE,
	UNUSED_01_PACKAGE_,
	UNUSED_02_PACKAGE_,
	UNUSED_03_PACKAGE_,
	UNUSED_04_PACKAGE_,
	UNUSED_05_PACKAGE_,
	UNUSED_06_PACKAGE_,
	UNUSED_07_PACKAGE_,
	UNUSED_08_PACKAGE_,
	UNUSED_09_PACKAGE_,
	UNUSED_10_PACKAGE_,
	UNUSED_11_PACKAGE_,
	UNUSED_12_PACKAGE_,
	UNUSED_13_PACKAGE_,
	UNUSED_14_PACKAGE_,
	UNUSED_15_PACKAGE_,
	UNUSED_16_PACKAGE_,
	UNUSED_17_PACKAGE_,
	UNUSED_18_PACKAGE_,
	UNUSED_19_PACKAGE_,
	CREDIT_MUSIC_PACKAGE,
	SISSKEL_PACKAGE,
	ORBENTER_PACKAGE,
	MENUBKG_PACKAGE,
	INTRO_PACKAGE,
	FINAL_PACKAGE,
	CREDIT_FONTS_PACKAGE,
	CREDITS_PACKAGE,
	JOYSTICK_ALPHA_PACKAGE,
};

