//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	HOMEWORLD_HELLO_1,
	HOMEWORLD_HELLO_2,
	whats_up_homeworld,
	GENERAL_INFO_HOMEWORLD_1,
	GENERAL_INFO_HOMEWORLD_2,
	i_demand_you_ally_homeworld0,
	i_demand_you_ally_homeworld1,
	i_demand_you_ally_homeworld2,
	i_demand_you_ally_homeworld3,
	ENEMY_MUST_DIE,
	at_least_help_us_homeworld,
	NO_HELP_ENEMY,
	give_info,
	NO_INFO_FOR_ENEMY,
	what_about_pkunk_royalist,
	PKUNK_ABSORBED_ROYALIST,
	HATE_PKUNK_ROYALIST,
	bye_homeworld,
	GOODBYE_AND_DIE_HOMEWORLD,
	SPACE_HELLO_1,
	SPACE_HELLO_2,
	SPACE_HELLO_3,
	SPACE_HELLO_4,
	whats_up_space_1,
	GENERAL_INFO_SPACE_1,
	whats_up_space_2,
	GENERAL_INFO_SPACE_2,
	whats_up_space_3,
	GENERAL_INFO_SPACE_3,
	whats_up_space_4,
	GENERAL_INFO_SPACE_4,
	i_demand_you_ally_space0,
	i_demand_you_ally_space1,
	i_demand_you_ally_space2,
	i_demand_you_ally_space3,
	WE_CANNOT_1,
	obligation,
	WE_CANNOT_2,
	at_least_help_us_space,
	SORRY_CANNOT,
	dishonor,
	HERES_A_HINT,
	what_about_pkunk_space,
	PKUNK_ABSORBED_SPACE,
	HATE_PKUNK_SPACE,
	bye_space,
	GO_IN_PEACE,
	GOODBYE_AND_DIE_SPACE,
	shofixti_alive_1,
	shofixti_alive_2,
	SEND_HIM_OVER_1,
	SEND_HIM_OVER_2,
	not_here,
	not_send,
	JUST_A_TRICK_1,
	JUST_A_TRICK_2,
	ok_send,
	WE_REVOLT,
	ROYALIST_SPACE_HELLO_1,
	ROYALIST_SPACE_HELLO_2,
	ROYALIST_HOMEWORLD_HELLO_1,
	ROYALIST_HOMEWORLD_HELLO_2,
	how_is_rebellion,
	ROYALIST_REBELLION_1,
	ROYALIST_REBELLION_2,
	sorry_about_revolution,
	ALL_YOUR_FAULT,
	bye_royalist,
	GOODBYE_AND_DIE_ROYALIST,
	name_1,
	name_2,
	name_3,
	name_40,
	name_41,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
