/////////////////////////////////////////////////////////////////////////////
// UrQuanMasters.rc
// 
// Resource script for Win32 builds
//

#include "../uqmversion.h"

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#ifdef _WIN32
LANGUAGE 0x09, 0x01	// LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32


/////////////////////////////////////////////////////////////////////////////
//
// Version
//

1   VERSIONINFO
 FILEVERSION UQM_MAJOR_VERSION,UQM_MINOR_VERSION,UQM_PATCH_VERSION,0
 PRODUCTVERSION UQM_MAJOR_VERSION,UQM_MINOR_VERSION,UQM_PATCH_VERSION,0
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", "See http://sc2.sourceforge.net\0"
            VALUE "CompanyName", "sc2.sourceforge.net\0"
            VALUE "FileDescription", "The Ur-Quan Masters main executable\0"
            VALUE "FileVersion", UQM_STRING_VERSION "\0"
            VALUE "InternalName", "uqm\0"
            VALUE "LegalCopyright", "(C) 1992-1993, 2002-2005 by respective authors\0"
#ifdef _DEBUG
            VALUE "OriginalFilename", "uqmdebug.exe\0"
#else
            VALUE "OriginalFilename", "uqm.exe\0"
#endif
            VALUE "ProductName", "The Ur-Quan Masters\0"
            VALUE "ProductVersion", UQM_STRING_VERSION "\0"
            VALUE "SpecialBuild", "Alpha\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
SDL_app            ICON    PRELOAD DISCARDABLE "ur-quan-icon-alpha.ico"
102                ICON    DISCARDABLE     "ur-quan-icon-std.ico"
103                ICON    DISCARDABLE     "ur-quan1.ico"
104                ICON    DISCARDABLE     "sis1.ico"
105                ICON    DISCARDABLE     "ur-quan2.ico"
106                ICON    DISCARDABLE     "kohr-ah1.ico"
107                ICON    DISCARDABLE     "starcon2.ico"
