#define CHENJESU_ICON_MASK_PMAP_ANIM 0x00200002L
#define CHENJESU_MICON_MASK_PMAP_ANIM 0x00200102L
#define CHENJESU_BIG_MASK_PMAP_ANIM 0x00400202L
#define CHENJESU_MED_MASK_PMAP_ANIM 0x00400302L
#define CHENJESU_SML_MASK_PMAP_ANIM 0x00400402L
#define SPARK_BIG_MASK_PMAP_ANIM 0x00400502L
#define SPARK_MED_MASK_PMAP_ANIM 0x00400602L
#define SPARK_SML_MASK_PMAP_ANIM 0x00400702L
#define DOGGY_BIG_MASK_PMAP_ANIM 0x00400802L
#define DOGGY_MED_MASK_PMAP_ANIM 0x00400902L
#define DOGGY_SML_MASK_PMAP_ANIM 0x00400a02L
#define CHENJESU_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


