#define ZOQFOTPIK_ICON_MASK_PMAP_ANIM 0x00200002L
#define ZOQFOTPIK_MICON_MASK_PMAP_ANIM 0x00200102L
#define ZOQFOTPIK_BIG_MASK_PMAP_ANIM 0x00400202L
#define ZOQFOTPIK_MED_MASK_PMAP_ANIM 0x00400302L
#define ZOQFOTPIK_SML_MASK_PMAP_ANIM 0x00400402L
#define SPIT_BIG_MASK_PMAP_ANIM 0x00400502L
#define SPIT_MED_MASK_PMAP_ANIM 0x00400602L
#define SPIT_SML_MASK_PMAP_ANIM 0x00400702L
#define STINGER_BIG_MASK_PMAP_ANIM 0x00400802L
#define STINGER_MED_MASK_PMAP_ANIM 0x00400902L
#define STINGER_SML_MASK_PMAP_ANIM 0x00400a02L
#define ZOQFOTPIK_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


