//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	INIT_HELLO,
	confused_by_hello,
	CONFUSED_RESPONSE,
	happy_by_hello,
	HAPPY_RESPONSE,
	miffed_by_hello,
	MIFFED_RESPONSE,
	FRIENDLY_SPACE_HELLO_1,
	FRIENDLY_SPACE_HELLO_2,
	FRIENDLY_SPACE_HELLO_3,
	FRIENDLY_SPACE_HELLO_4,
	FRDLY_HOMEWORLD_HELLO_1,
	FRDLY_HOMEWORLD_HELLO_2,
	FRDLY_HOMEWORLD_HELLO_3,
	FRDLY_HOMEWORLD_HELLO_4,
	whats_up_1,
	whats_up_2,
	GENERAL_INFO_1,
	GENERAL_INFO_2,
	GENERAL_INFO_3,
	GENERAL_INFO_4,
	why_you_here,
	LEARN_THINGS,
	what_things,
	THESE_THINGS,
	why_do_it,
	DO_IT_BECAUSE,
	give_me_info_1,
	ARILOU_HINTS_1,
	give_me_info_2,
	ARILOU_HINTS_2,
	ARILOU_HINTS_3,
	ARILOU_HINTS_4,
	bye_friendly_space,
	GOODBYE_FRIENDLY_SPACE,
	GOT_PART_YET_1,
	GOT_PART_YET_2,
	INIT_ANGRY_HWLD_HELLO,
	invaders_from_mars,
	HAD_OUR_REASONS,
	bug_eyed_fruitcakes,
	WE_NEVER_FRIENDS,
	ok_lets_be_friends,
	NO_ALLY_BUT_MUCH_GIVE,
	why_should_i_trust,
	TRUST_BECAUSE,
	what_about_interference,
	INTERFERENCE_NECESSARY,
	i_just_like_to_leave,
	SORRY_NO_LEAVE,
	what_about_war,
	ABOUT_WAR,
	what_about_urquan,
	ABOUT_URQUAN,
	best_if_i_killed_you,
	WICKED_HUMAN,
	what_did_on_earth,
	DID_THIS,
	why_did_this,
	IDF_PARASITES,
	tell_more,
	NOT_NOW,
	umgah_acting_weird,
	learned_about_umgah,
	WELL_GO_CHECK,
	NO_NEWS_YET,
	UMGAH_UNDER_COMPULSION,
	what_do_now,
	GO_FIND_OUT,
	tell_arilou_about_tpet,
	BAD_NEWS_ABOUT_TPET,
	what_do_about_tpet,
	DANGEROUS_BUT_USEFUL,
	what_give_me,
	ABOUT_PORTAL,
	what_about_tpet,
	ABOUT_TPET,
	about_portal_again,
	PORTAL_AGAIN,
	got_it,
	CLEVER_HUMAN,
	GIVE_PORTAL,
	bye_friendly_homeworld,
	GOODBYE_FRDLY_HOMEWORLD,
	HOSTILE_GOODBYE_1,
	HOSTILE_GOODBYE_2,
	HOSTILE_GOODBYE_3,
	HOSTILE_GOODBYE_4,
	ANGRY_SPACE_HELLO_1,
	ANGRY_SPACE_HELLO_2,
	lets_fight,
	NO_FIGHT,
	im_sorry,
	APOLOGIZE_AT_HOMEWORLD,
	bye_angry_space,
	GOODBYE_ANGRY_SPACE,
	OUT_TAKES,
};

#endif /* _STRINGS_H */
