#define ANDROSYNTH_ICON_MASK_PMAP_ANIM 0x00200002L
#define ANDROSYNTH_MICON_MASK_PMAP_ANIM 0x00200102L
#define ANDROSYNTH_BIG_MASK_PMAP_ANIM 0x00400202L
#define ANDROSYNTH_MED_MASK_PMAP_ANIM 0x00400302L
#define ANDROSYNTH_SML_MASK_PMAP_ANIM 0x00400402L
#define BUBBLE_BIG_MASK_PMAP_ANIM 0x00400502L
#define BUBBLE_MED_MASK_PMAP_ANIM 0x00400602L
#define BUBBLE_SML_MASK_PMAP_ANIM 0x00400702L
#define BLAZER_BIG_MASK_PMAP_ANIM 0x00400802L
#define BLAZER_MED_MASK_PMAP_ANIM 0x00400902L
#define BLAZER_SML_MASK_PMAP_ANIM 0x00400a02L
#define ANDROSYNTH_CAPT_MASK_PMAP_ANIM 0x00400b02L


