//Copyright Paul Reiche, Fred Ford. 1992-2002

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _STRINGS_H
#define _STRINGS_H

enum
{
	NULL_PHRASE,
	TELL_US_ABOUT_WORLD,
	BYE_AND_DIE_HOMEWORLD,
	RAMBLE_1,
	RAMBLE_2,
	RAMBLE_3,
	RAMBLE_4,
	RAMBLE_5,
	RAMBLE_6,
	RAMBLE_7,
	RAMBLE_8,
	RAMBLE_9,
	RAMBLE_10,
	RAMBLE_11,
	RAMBLE_12,
	RAMBLE_13,
	RAMBLE_14,
	RAMBLE_15,
	RAMBLE_16,
	RAMBLE_17,
	RAMBLE_18,
	RAMBLE_19,
	RAMBLE_20,
	RAMBLE_21,
	RAMBLE_22,
	RAMBLE_23,
	RAMBLE_24,
	RAMBLE_25,
	RAMBLE_26,
	RAMBLE_27,
	RAMBLE_28,
	RAMBLE_29,
	RAMBLE_30,
	RAMBLE_31,
	RAMBLE_32,
	question_1,
	question_2,
	question_3,
	question_4,
	question_5,
	question_6,
	question_7,
	question_8,
	question_9,
	question_10,
	question_11,
	question_12,
	question_13,
	question_14,
	question_15,
	question_16,
	bye_space,
	BYE_AND_DIE_SPACE,
	gonna_die,
	insult_1,
	insult_2,
	insult_3,
	insult_4,
	insult_5,
	insult_6,
	insult_7,
	insult_8,
	come_in_peace,
	HELLO_HOMEWORLD_1,
	HELLO_HOMEWORLD_2,
	HELLO_HOMEWORLD_3,
	HELLO_HOMEWORLD_4,
	HELLO_HOMEWORLD_5,
	HELLO_HOMEWORLD_6,
	HELLO_HOMEWORLD_7,
	HELLO_HOMEWORLD_8,
	HELLO_SPACE_1,
	HELLO_SPACE_2,
	HELLO_SPACE_3,
	HELLO_SPACE_4,
	HELLO_SPACE_5,
	HELLO_SPACE_6,
	HELLO_SPACE_7,
	HELLO_SPACE_8,
	lets_be_friends,
	came_to_homeworld,
	submit_to_us,
	bye_sun_device,
	GOODBYE_SUN_DEVICE,
	RESPONSE_1,
	RESPONSE_2,
	RESPONSE_3,
	clue_1,
	clue_2,
	clue_3,
	what_about_shattered,
	ABOUT_SHATTERED,
	HELLO_SUN_DEVICE_WORLD_1,
	HELLO_SUN_DEVICE_WORLD_2,
	whats_up_sun_device,
	GENERAL_INFO_SUN_DEVICE,
	like_to_land,
	NEVER_LET_LAND,
	bye_homeworld,
	i_have_a_cunning_plan,
	DIE_LIAR,
	how_goes_implanting,
	UNFORSEEN_DELAYS,
	DIE_THIEF,
	DIE_THIEF_AGAIN,
	GOODBYE_AND_DIE,
	AMBUSH_TAIL,
	RAMBLE_TAIL,
	WE_GO_TO_IMPLANT,
	WONT_FALL_FOR_TRICK,
};

#endif /* _STRINGS_H */
