#define KOHR_AH_ICON_MASK_PMAP_ANIM 0x00200002L
#define KOHR_AH_MICON_MASK_PMAP_ANIM 0x00200102L
#define KOHR_AH_BIG_MASK_PMAP_ANIM 0x00400202L
#define KOHR_AH_MED_MASK_PMAP_ANIM 0x00400302L
#define KOHR_AH_SML_MASK_PMAP_ANIM 0x00400402L
#define BUZZSAW_BIG_MASK_PMAP_ANIM 0x00400502L
#define BUZZSAW_MED_MASK_PMAP_ANIM 0x00400602L
#define BUZZSAW_SML_MASK_PMAP_ANIM 0x00400702L
#define GAS_BIG_MASK_PMAP_ANIM 0x00400802L
#define GAS_MED_MASK_PMAP_ANIM 0x00400902L
#define GAS_SML_MASK_PMAP_ANIM 0x00400a02L
#define KOHR_AH_CAPTAIN_MASK_PMAP_ANIM 0x00400b02L


