#define STARCON_FONT 0x03800003L
#define TINY_FONT 0x03800103L
#define MICRO_FONT 0x03c00203L
#define LANDER_FONT 0x11400303L
#define PLAYER_FONT 0x17a00403L

#define PT13AA_FONT 0x24000503L
#define PT17AA_FONT 0x24000603L
#define PT45AA_FONT 0x24000703L

